# Copyright (c) 2002-2003 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2002 Prfecture du Bas-Rhin, France
# Copyright (c) 2002 CIRB, Belgique
# Authors: Florent Guillaume <mailto:fg@nuxeo.com>
#          Julien Jalon <mailto:jj@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: cpsinstall.py,v 1.68 2003/08/09 07:12:00 fermigier Exp $

#
# This installs the necessary components for CPS.
# It must be run inside a fresh CMF.
#

# When adding a new type, add it to:
# - list of portal types
# - workflows
# - Workgroup allowed content types

import os
import sys
from random import randrange
from Acquisition import aq_base
from DateTime.DateTime import DateTime
from AccessControl import getSecurityManager
from App.Extensions import getPath
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.Expression import Expression

this_cps_version = 3

def packageHome(name):
    """Returns path to Products.name"""
    m = sys.modules['Products.%s' % name]
    return (m.__path__[0])

cps_home = packageHome('NuxCPS')
zexpdir = os.path.join(cps_home, 'Install')

def tryimport(container, name, suffix='zexp', zexpdir=zexpdir, pr=None):
    zexppath = os.path.join(zexpdir, '%s.%s' % (name, suffix))
    container._importObjectFromFile(zexppath)
    pr(" Import of %s.%s" % (name, suffix))


def setperms(object, perms, pr=None):
    for perm, roles in perms.items():
        acquire = type(roles) == type([])
        object.manage_permission(perm, roles, acquire)
        pr("  Permission %s" % perm)


def cpsinstall(self):
    """
    Cleanup and change stuff from a fresh CMF.
    Do this only once...
    """
    log = []
    pr = log.append

    pr("Start cleaning")
    pr("")

    portal = self.portal_url.getPortalObject()
    pr("Portal found in %s" % '/'.join(portal.getPhysicalPath()))

    portal_actions = portal.portal_actions
    pr("Cleaning unused action providers in portal_action")
    providers = ['portal_syndication', 'portal_undo']
    for prov in providers:
        if prov in portal_actions.listActionProviders():
            portal_actions.manage_aproviders(chosen=[prov], del_provider=1)
            pr(" Deleting action provider %s" % prov)

    pr("Cleaning actions")
    actiondelmap = {
        'portal_actions': ['folderContents', 'folder_contents'],
        'portal_membership': ['preferences', 'addFavorite',
                              'mystuff', 'favorites'],
        'portal_registration': ['join'],
        'portal_syndication': ['syndication'],
        }
    for tool, actionids in actiondelmap.items():
        actions = list(portal[tool]._actions)
        for ac in actions:
            id = ac.id
            if id in actionids:
                if ac.visible:
                    ac.visible = 0
                    pr(" Deleting %s: %s" % (tool, id))
        portal[tool]._actions = actions

    pr("Renaming actions")
    actionrenmap = {
        'portal_discussion': {'Reply': '_action_comment_'},
        'portal_membership': {'Login': '_action_login_',
                              'Log out': '_action_logout_',
                              },
        'portal_properties': {'Reconfigure Portal': '_action_portalconfiguration_'},
        }
    for tool, renames in actionrenmap.items():
        keys = renames.keys()
        actions = list(portal[tool]._actions)
        for ac in actions:
            title = ac.title
            if title in keys:
                newtitle = renames[title]
                ac.title = newtitle
                pr(" Renaming %s from %s to %s" % (tool, title, newtitle))
        portal[tool]._actions = actions

    pr("Renaming Members")
    portal.Members.title = "_str_Members_"
    pr("")
    pr("End cleaning")
    pr("Setting CPS build version to %s" % (this_cps_version, ))
    portal._cpsversion = this_cps_version
    return '\n'.join(log)


def cpsupdate(self, langs_list=None):
    log = []
    pr = log.append
    def primp(pr=pr):
        pr(" !!! Can migrate that component !!!")
    def prok(pr=pr):
        pr(" Already correctly installed")

    pr("Starting update")
    pr("")

    # find portal root
    portal = self.portal_url.getPortalObject()
    pr("Portal found in %s" % '/'.join(portal.getPhysicalPath()))

    installername = getSecurityManager().getUser().getUserName()
    pr("Current user: %s" % installername)

    cps_version = getattr(aq_base(portal), '_cpsversion', None)
    if cps_version is None:
        cps_version = 0
        portal._cpsversion = 0

    def portalhas(id, portal=portal):
        return id in portal.objectIds()

    # initial cleanup XXX to execute only after a fresh CMF
    #pr(cpscleanup(self))

    # update catalog indexes
    pr("Verifying portal_catalog indexes")
    indexes = {
        'LevelIsPublic': 'FieldIndex',
        'LevelReaders': 'KeywordIndex',
        'LevelReviewers': 'KeywordIndex',
        'ModificationDate': 'FieldIndex',
        'PendingCategories': 'KeywordIndex',
        'PendingSignatures': 'KeywordIndex',
        'PublicCategories': 'KeywordIndex',
        'portal_type': 'FieldIndex',
        'version_state': 'FieldIndex',
        'dateiso': 'FieldIndex',
        'is_stable': 'FieldIndex',
    }
    metadata = [
        'id',
        'CreationDate',
        'ModificationDate',
        'PendingCategories',
        'PublicCategories',
        'getIconRelative',
        'isPortalContent',
        'isPrincipiaFolderish',
        'isTitleDynamic',
        'levelkey',
    ]
    catalog = portal.portal_catalog
    for ix, typ in indexes.items():
        if ix in catalog.Indexes.objectIds():
            pr(" %s: ok" % ix)
        else:
            prod = catalog.Indexes.manage_addProduct['PluginIndexes']
            constr = getattr(prod, 'manage_add%s' % typ)
            constr(ix)
            pr(" %s: added" % ix)
    pr("Verifying portal_catalog metadatas")
    for md in metadata:
        if md in catalog.schema():
            pr(" %s: ok" % md)
        else:
            catalog.addColumn(md)
            pr(" %s: added" % md)

    pr("Verifying roles")
    already = portal.valid_roles()
    for role in ('Reader', 'WorkgroupManager', 'WorkgroupMember', 'WorkgroupVisitor'):
        if role not in already:
            portal._addRole(role)
            pr(" Add role %s" % role)

    # replace user folder with a User Folder With Groups
    pr("Verifying User Folder")
    if portalhas('acl_users'):
        aclu = portal.acl_users
        if aclu.meta_type == 'User Folder With Groups':
            prok()
        else:
            usernames = aclu.getUserNames()
            if usernames:
                pr(" !!! User Folder already contains users")
                primp()
            else:
                portal.manage_delObjects(['acl_users'])
    if not portalhas('acl_users'):
        pr(" Creating User Folder With Groups")
        portal.manage_addProduct['NuxUserGroups'].addUserFolderWithGroups()

    # create index_html for portal redirection
    pr("Verifying global index_html")
    if portalhas('index_html'):
        if portal.index_html.meta_type != 'DTML Method':
            pr(" !!! index_html is not a DTML Method")
            primp()
        else:
            prok()
    else:
        pr(" Creating index_html")
        portal.manage_addProduct['OFSP'].manage_addDTMLMethod('index_html')
        doc = portal['index_html']
        doc.munge('<dtml-var portal>')

    # create portal_preferences tool
    pr("Verifying portal_preferences")
    if portalhas('portal_preferences'):
        prok()
    else:
        pr(" Creating portal_preferences")
        portal.manage_addProduct["NuxCPSPreferences"].manage_addTool('Portal Preferences Tool')

    # create portal_boxes tool
    pr("Verifying portal_boxes")
    if portalhas('portal_boxes'):
        prok()
    else:
        pr(" Creating portal_boxes")
        portal.manage_addProduct['NuxPortal'].manage_addTool('Portal Boxes Tool')

    # create portal_messages tool
    pr("Verifying portal_messages")
    if portalhas('portal_messages'):
        has_mcat = 1
        prok()
    else:
        has_mcat = 0
        pr(" Creating portal_messages")
        portal.manage_addProduct['NuxCPSMessageCatalog'].manage_addTool('Message Catalog Tool')
        portal.portal_messages.manage_delLanguages(['en'])
    
    mcat = portal.portal_messages

    # importing .po files
    pr("Checking available languages")
    popath = os.path.join(cps_home, 'Install')
    if popath is None:
        pr(" !!! Unable to find .po dir")
    else:
        pr(" Checking installable languages")
        langs = []
        for file in os.listdir(popath):
            if file.endswith('.po'):
                lang = '.'.join(file.split('.')[:-1])
                pr("  Found language %s" % (lang, ))
                langs.append(lang)
        if has_mcat:
            pr(" Checking available languages in portal_messages")
        avail_langs = mcat.get_languages()
        if has_mcat:
            pr("  Found %s" % (', '.join(avail_langs), ))

        # install only needed langs
        if has_mcat and langs_list is None:
            langs = [lang for lang in avail_langs if lang in langs]
        elif langs_list is not None:
            langs = [lang for lang in langs_list if lang in langs]
        else:
            langs = langs

        if langs_list is not None:
            pr(" Only using %s" % (', '.join(langs_list), ))

        append_langs = [lang for lang in langs if lang not in avail_langs]
        if append_langs:
            pr(" Will add %s to portal_messages" % (', '.join(append_langs), ))
        for lang in append_langs:
            pr("  Adding %s" % (lang, ))
            mcat.manage_addLanguage(lang)
        if langs:
            pr(" Will import dictionaries for %s" % (', '.join(langs), ))
        for lang in langs:
            pr("  Importing %s.po" % (lang, ))
            lang_po_path = os.path.join(popath, lang+'.po')
            lang_file = open(lang_po_path)
            mcat.manage_import(lang, lang_file)
        if not has_mcat and langs:
            pr(" Setting default language to %s" % (langs[0], ))
            mcat.manage_changeDefaultLang(langs[0])
    
    # create portal_metadirectories tool
    pr("Verifying portal_metadirectories")
    if portalhas('portal_metadirectories'):
        prok()
    else:
        pr(" Creating portal_metadirectories")
        portal.manage_addProduct['NuxMetaDirectories'].manage_addTool('CMF MetaDirectories Tool')
    pr("Verifying directories")
    dirtool = portal.portal_metadirectories
    directories = (('members', '_dir_Users_', 'Members Directory'), ('groups', '_dir_Groups_', 'Groups Directory'))
    for id, title, meta_type in directories:
        if id in dirtool.objectIds():
            pr(' %s seems correctly installed' % id)
        else:
            pr(' Creating %s' % id)
            getattr(dirtool.manage_addProduct['NuxMetaDirectories'], 'manage_add%s' % ''.join(meta_type.split()))(id, title)
            if id == 'members':
                pr(' Synchronizing with MemberData')
                dirtool.members.syncSchemaAndMemberData()

    # create portal_hierarchies tool
    pr("Verifying portal_hierarchies")
    if portalhas('portal_hierarchies'):
        prok()
    else:
        pr(" Creating portal_hierarchies")
        portal.manage_addProduct['NuxHierarchies'].manage_addTool('Hierarchies Tool')

    # create portal_version_catalog tool
    pr("Verifying portal_version_catalog")
    if portalhas('portal_version_catalog'):
        prok()
    else:
        pr(" Creating portal_version_catalog")
        portal.manage_addProduct['VersionManager'].manage_addTool('CMF Catalog')

    # create portal_workflow_schema tool
    pr("Verifying portal_workflow_schema")
    if portalhas('portal_workflow_schema'):
        prok()
    else:
        pr(" Creating portal_workflow_schema")
        portal.manage_addProduct['WorkflowSchema'].manage_addTool('Workflow Schema Tool')

    # create portal_subscription tool
    pr("Verifying portal_subscription")
    if portalhas('portal_subscription'):
        prok()
    else:
        pr(" Creating portal_subscription tool")
        portal.manage_addProduct['NuxCPSSubscription'].manage_addTool(
            'Portal Subscription Tool')

    # create portal_events tool
    pr("Verifying portal_events")
    if portalhas('portal_events'):
        prok()
    else:
        pr(" Creating portal_events tool")
        portal.manage_addProduct['Event'].manage_addEventService(
            id="portal_events", title="Event Service")

    # create portal_scheduler tool
    pr("Verifying portal_scheduler")
    if portalhas('portal_scheduler'):
        prok()
    else:
        pr(" Creating portal_scheduler tool")
        portal.manage_addProduct['Scheduler'].manage_addScheduler(
            id="portal_scheduler", title="Portal Scheduler", 
            filter_data="scheduler")

    # skins
    pr("Verifying skins")

    pr(" Testing Optionnal packages")
    ## RTE
    try:
        import Products.NuxRichTextEditor
        nuxrte = 1
        pr("  NuxRTE is present.")
    except ImportError:
        nuxrte = 0
        pr("  NuxRTE is missing. Won't try to install it.")

    ## NuxCPSForum
    try:
        import Products.NuxCPSForum
        nuxcpsforum = 1
        pr("  NuxCPSForum is present.")
    except ImportError:
        nuxcpsforum = 0
        pr("  NuxCPSForum is missing. Won't try to install it.")

    ## NuxCPSFlexibleDocument
    try:
        import Products.NuxCPSFlexibleDocument
        nuxcpsflexibledocument = 1
        pr("  NuxCPSFlexibleDocument is present.")
    except ImportError:
        nuxcpsflexibledocument = 0
        pr("  NuxCPSFlexibleDocument is missing. Won't try to install it.")

    ## NuxGroupCalendar

    try:
        import Products.NuxGroupCalendar
        nuxgroupcalendar = 1
        pr("  NuxGroupCalendar is present.")
    except ImportError:
        nuxgroupcalendar = 0
        pr("  NuxGroupCalendar is missing. Won't try to install it.")

    skins = ('nuxcpsdefault', 'nuxcps', 'nuxportal', 'nuxmetadirectories',
             'nuxworkgroup', 'nuxcpsdocuments', 'nuxdocument',
             'nuxhierarchies', 'workflowschema')
    paths = {
        'nuxcpsdefault': 'Products/NuxCPS/default_skins',
        'nuxcps': 'Products/NuxCPS/skins',
        'nuxportal': 'Products/NuxPortal/skins',
        'nuxmetadirectories': 'Products/NuxMetaDirectories/skins',
        'nuxworkgroup': 'Products/NuxWorkgroup/skins',
        'nuxcpsdocuments': 'Products/NuxCPSDocuments/skins',
        'nuxdocument': 'Products/NuxDocument/skins',
        'nuxhierarchies': 'Products/NuxHierarchies/skins',
        'workflowschema': 'Products/WorkflowSchema/skins',
    }

    if nuxrte:
        skins = ('nuxrte', ) + skins
        paths['nuxrte'] = 'Products/NuxRichTextEditor/skins'

    if nuxcpsforum:
        skins = ('nuxcpsforum_pleiade', 'nuxcpsforum_standard') + skins
        paths['nuxcpsforum_standard'] = 'Products/NuxCPSForum/skins/standard'
        paths['nuxcpsforum_pleiade'] = 'Products/NuxCPSForum/skins/pleiade'

    if nuxcpsflexibledocument:
        skins = ('nuxcpsflexibledocument', ) + skins
        paths['nuxcpsflexibledocument'] = \
            'Products/NuxCPSFlexibleDocument/skins'

    if nuxgroupcalendar:
        skins = ('nuxgroupcalendar', ) + skins
        paths['nuxgroupcalendar'] = 'Products/NuxGroupCalendar/skins'

    for skin in skins:
        path = paths[skin]
        path = path.replace('/', os.sep)
        pr(" FS Directory View '%s'" % skin)
        if skin in portal.portal_skins.objectIds():
            dv = portal.portal_skins[skin]
            oldpath = dv.getDirPath()
            if oldpath == path:
                prok()
            else:
                pr("  Correctly installed, correcting path")
                dv.manage_properties(dirpath=path)
        else:
            portal.portal_skins.manage_addProduct['CMFCore'].manage_addDirectoryView(filepath=path, id=skin)
            pr("  Creating skin")
    allskins = portal.portal_skins.getSkinPaths()
    for skin_name, skin_path in allskins:
        if skin_name != 'Basic':
            continue
        path = [x.strip() for x in skin_path.split(',')]
        path = [x for x in path if x not in skins] # strip all
        if path and path[0] == 'custom':
            path = path[:1] + list(skins) + path[1:]
        else:
            path = list(skins) + path
        npath = ', '.join(path)
        portal.portal_skins.addSkinSelection(skin_name, npath)
        pr(" Fixup of skin %s" % skin_name)

    # portal types
    pr("Verifying portal types")
    containers = ('Workgroup', 'Reviewed Hierarchy Level')
    ptypes = {
        'CatNav Box': {'tin': 'NuxPortal: CatNav Box',
                        'containers': ()},
        'Contact Document': {'tin': 'NuxCPSDocuments: Contact Document',
                        'containers': ('Workgroup')},
        'Discussion Item': {'tin': 'CMFDefault: Discussion Item',
                        'containers': ()},
        'Doc Filter Box': {'tin': 'NuxPortal: Doc Filter Box',
                        'containers': ()},
        'FAQ Document': {'tin': 'NuxCPSDocuments: FAQ Document',
                        'containers': ('Workgroup')},
        'File Document': {'tin': 'NuxCPSDocuments: File Document',
                        'containers': ('Workgroup')},
        'Help Document': {'tin': 'CMFDefault: Document',
                        'containers': ()},
        'Help Folder': {'tin': 'CMFDefault: Skinned Folder',
                        'containers': ()},
        'Mailing List': {'tin': 'NuxCPSDocuments: Mailing List',
                         'containers': ('Workgroup', 'Reviewed Hierarchy Level')},
        'Newsletter': {'tin': 'NuxCPSDocuments: Newsletter',
                       'containers': ('Reviewed Hierarchy Level','Workgroup')},
        'News Document': {'tin': 'NuxCPSDocuments: News Document',
                          'containers': ('Workgroup')},
        'Reviewed Hierarchy Level': {'tin': 'NuxHierarchies: Reviewed Hierarchy Level',
                        'containers': ('Reviewed Hierarchy Level')},
        'Text Box': {'tin': 'NuxPortal: Text Box',
                        'containers': ()},
        'Text Document': {'tin': 'NuxCPSDocuments: Text Document',
                        'containers': ('Workgroup')},
        'URL Document': {'tin': 'NuxCPSDocuments: URL Document',
                        'containers': ('Workgroup')},
        'WGNav Box': {'tin': 'NuxPortal: WGNav Box',
                        'containers': ()},
        'Workgroup': {'tin': 'NuxWorkgroup: Workgroup',
                        'containers': ('Workgroup')},
        'Skinner Document': {'tin': 'NuxCPSDocuments: Skinner Document',
                        'containers': ('Reviewed Hierarchy Level')},
        'Workflow Schema': {'tin': 'WorkflowSchema: Workflow Schema',
                        'containers': ()},
        'Workflow Schema State': {
            'tin': 'WorkflowSchema: Workflow Schema State',
            'containers': ()
        },
        'Workflow Schema Event Container': {
            'tin': 'WorkflowSchema: Workflow Schema Event Container',
            'containers': ()
        },
        'Workflow Schema Mail Event': {
            'tin': 'WorkflowSchema: Workflow Schema Mail Event',
            'containers': ()
        },
        'Workflow Schema Publication Switch Event': {
            'tin': 'WorkflowSchema: Workflow Schema Publication Switch Event',
            'containers': ()
        },
        'Workflow Schema Condition Container': {
            'tin': 'WorkflowSchema: Workflow Schema Condition Container',
            'containers': ()
        },
        'Workflow Schema Substate Condition': {
            'tin': 'WorkflowSchema: Workflow Schema Substate Condition',
            'containers': ()
        },
        'Workflow Schema Signature Condition': {
            'tin': 'WorkflowSchema: Workflow Schema Signature Condition',
            'containers': ()
        },
    }

    if nuxcpsforum:
        ptypes.update(
            {
            'Forum': {'tin': 'NuxCPSForum: Forum',
                        'containers': ('Workgroup')},
            'ThreadSnapShot': {'tin': 'NuxCPSForum: ThreadSnapShot',
                        'containers': ('Workgroup')},
            'Post': {'tin': 'NuxCPSForum: Post',
                        'containers': ()},
            }
        )

    if nuxcpsflexibledocument:
        ptypes.update(
            {
            'Flexible Document': {'tin': 'NuxCPSFlexibleDocument: Flexible Document',
                        'containers': ('Workgroup')},
            }
        )

    if nuxgroupcalendar:
        ptypes.update(
            {
            'Event': {'tin': 'NuxGroupCalendar: Event',
                        'containers': ()},
            'Calendar': {'tin': 'NuxGroupCalendar: Calendar',
                        'containers': ()},
            'Calendars': {'tin': 'NuxGroupCalendar: Calendars',
                        'containers': ()},
            }
        )

    ptypes_ordered = [
        'Reviewed Hierarchy Level',
        'File Document',
        'Text Document',
        'News Document',
        'FAQ Document',
        'Contact Document',
        'URL Document',
        'Mailing List',
        'Newsletter',
        'Workgroup',
        'Discussion Item',
        'Help Document',
        'Help Folder',
        'Doc Filter Box',
        'Text Box',
        'CatNav Box',
        'WGNav Box',
        'Skinner Document',
        'Workflow Schema',
        'Workflow Schema State',
        'Workflow Schema Event Container',
        'Workflow Schema Mail Event',
        'Workflow Schema Publication Switch Event',
        'Workflow Schema Condition Container',
        'Workflow Schema Substate Condition',
        'Workflow Schema Signature Condition',
    ]

    if nuxcpsforum:
        ptypes_ordered = ptypes_ordered[:8] \
            + ['Forum', 'ThreadSnapShot', 'Post'] + ptypes_ordered[8:]

    if nuxcpsflexibledocument:
        ptypes_ordered.append('Flexible Document')

    if nuxgroupcalendar:
        ptypes_ordered.extend(['Event', 'Calendar', 'Calendars'])

    # check ptypes_ordered and ptypes
    in_ptypes_ordered = [a for a in ptypes_ordered if a not in ptypes.keys()]
    in_ptypes = [a for a in ptypes.keys() if a not in ptypes_ordered]
    if in_ptypes_ordered:
        s = '%s are in ptypes_ordered but not in ptypes' % (
            in_ptypes_ordered,
        )
        raise s
    if in_ptypes:
        s = '%s are in ptypes but not in ptypes_ordered' % (
            in_ptypes,
        )
        raise s
    if len(ptypes.keys()) != len(ptypes_ordered):
        from pprint import pformat
        raise 'BUG:\n%s' % pformat((ptypes.keys(), ptypes_ordered))


    containers_dict = {}
    # XXX don't forget to change Workgroup allowed_content_types too XXX !!!
    ttool = portal.portal_types
    alldocs = []
    for ptype in ptypes_ordered:
        # XXX: This try / except is a hack. Refactor this fscking mess !
        try:
            dic = ptypes[ptype]
            pr(" Type '%s'" % ptype)
            if ptype in ttool.objectIds():
                ttool.manage_delObjects([ptype])
                pr("  Deleted")
            if ptype not in ttool.objectIds():
                tin = dic['tin']
                amt = 'Factory-based Type Information'
                try:
                    ttool.manage_addTypeInformation(add_meta_type=amt, id=ptype,
                                                    typeinfo_name=tin)
                    pr("  Installation")
                except:
                    pr("  !!! Installation failed")
            else:
                pr("  Ok")
            ti = ttool[ptype]
            allowed_containers = dic['containers']
            if type(allowed_containers) is type(''):
                allowed_containers = (allowed_containers, )
            for container in allowed_containers:
                if container not in containers:
                    raise 'BUG while installing %s (container=%s)' % (ptype, container)
                allowed_types = containers_dict.get(container, [])
                allowed_types.append((ptype, ti.getId()))
                containers_dict[container] = allowed_types
        except:
                pr("  !!! Installation of type %s failed" % ptype)


    if not ttool['Discussion Item'].title:
        ttool['Discussion Item'].title = 'Comment'
        pr(" Discussion Item: title=Comment")

    # add filtered types to Workgroup
    pr(" Verifying allowed types in containers")
    for container in containers:
        container_ti = ttool[container]
        container_types = containers_dict[container]
        pr("  %s :" % (container, ))
        allowed = list(container_ti.allowed_content_types)
        for pt, ct in container_types:
            if ct not in allowed:
                allowed.append(ct)
                pr("   Add %s" % ct)
            else:
                pr("   %s Already allowed" % ct)
        container_ti.allowed_content_types = tuple(allowed)

    # discussion item: add 'delete' action
    ti = ttool['Discussion Item']
    if ti.getActionById('delete', None) is None:
        pr(" Discussion Item: add action 'delete'")
        ti.addAction(id='delete',
                     name='_action_delete_',
                     action='discussionitem_delete',
                     permission='Delete replies',
                     category='object',
                     visible=1,
                     )
    else:
        pr(" Discussion Item: 'delete' action already there")
    if ti.getActionById('view', None) is None:
        pr(" Discussion Item: 'view' action not present")
    else:
        pr(" Discussion Item: rename action 'view'")
        disc_actions = ti.getActions()
        for action in disc_actions:
            if action.has_key('id'):
                if action['id'] == 'view':
                    pr("  %s -> _action_view_" % (action['name'], ))
                    action['name'] = '_action_view_'

    # help folder
    ti = ttool['Help Folder']
    ti.manage_changeProperties(filter_content_types=1,
                               allowed_content_types=('Help Document',)
                               )

    # workflows
    # XXX probably all 'isdocument' can have a std workflow
    wfs = {
        'CatNav Box': '',
        'Contact Document': 'publication_workflow, version_support',
        'Discussion Item': '',
        'Doc Filter Box': '',
        'Document': '',
        'FAQ Document': 'publication_workflow, version_support',
        'File Document': 'publication_workflow, version_support',
        'Folder': '',
        'Help Document': '',
        'Help Folder': '',
        'Mailing List': '',
        #'Newsletter': '',
        'News Document': 'publication_workflow, version_support',
        'Reviewed Hierarchy Level': '',
        'Text Box': '',
        'Text Document': 'publication_workflow, version_support',
        'URL Document': 'publication_workflow, version_support',
        'WGNav Box': '',
        'Workgroup': '',
        'Skinner Document': '',
    }

    if nuxcpsforum:
        wfs['ThreadSnapShot'] = 'publication_workflow'

    if nuxcpsflexibledocument:
        wfs['Flexible Document'] = 'publication_workflow, version_support'

    wftool = portal.portal_workflow
    pr("Installing workflow schemas")
    for pt, chain in wfs.items():
        wftool.setChainForPortalTypes([pt], chain)
    wftool.setDefaultChain('')
    # import des workflow
    pr("Importing workflows")
    wfids = wftool.objectIds()
    for wfid in ('publication_workflow', 'version_support'):
        if wfid in wfids:
            pr(" Reinstalling %s" % wfid)
            wftool.manage_delObjects([wfid])
        tryimport(wftool, wfid, pr=pr)
    if 'default_workflow' in wfids:
        pr("Deleting default_workflow")
        wftool.manage_delObjects(['default_workflow'])

    if nuxcpsforum:
        wfid = 'post_workflow'
        if wfid in wfids:
            pr(" Reinstalling %s" % (wfid, ))
            wftool.manage_delObjects([wfid])
        tryimport(wftool, wfid, zexpdir=os.path.join(
            packageHome('NuxCPSForum'), 'Install'
        ), pr=pr)

    pr("Verifying new actions")
    actionadd = {
        'portal_actions': [ActionInformation(id='print',
                                             title='_action_print_',
                                             action=Expression(
        text='string:${object_url}/view?pp=1'),
                                             condition=Expression(
        text='object/isPortalContent | nothing'),
                                             permissions=('View',),
                                             category='object',
                                             visible=0,
                                             ),
                           ActionInformation(id='directories',
                                             title='_action_directories_',
                                             action=Expression(
        text='string:${portal_url}/directories'),
                                             permissions=('View',),
                                             category='global',
                                             visible=1,
                                             ),
                           ActionInformation(id='globalboxes',
                                             title='_action_globalboxes_',
                                             action=Expression(
        text='string:${portal_url}/boxes_config'),
                                             permissions=('Manage properties',),
                                             category='global',
                                             visible=1,
                                             ),
                           ActionInformation(id='personalboxes',
                                             title='_action_personalboxes_',
                                             action=Expression(
        text='string:${portal_url}/boxes_config?perso=1'),
                                             condition='member',
                                             category='user',
                                             visible=1,
                                             ),
                           ActionInformation(id='help',
                                             title='_action_help_',
                                             action=Expression(
        text='string:help?where=portal'),
                                             condition=Expression(
        text='python: object is portal'),
                                             permissions=('View',),
                                             category='global',
                                             visible=0,
                                             ),
                           ],
        'portal_membership': [ActionInformation(id='members',
                                             title='_action_members_',
                                             action=Expression(
        text='string:${portal_url}/Members/'),
                                             permissions=('Manage users',),
                                             category='global',
                                             visible=1),
                            ],
        }

    if nuxgroupcalendar:
        pacts = actionadd['portal_actions']
        pacts.extend([
            ActionInformation(
                id='calendars',
                title='_action_calendars_',
                action=Expression(text='string:${portal_url}/Calendars'),
                permissions=('View',),
                category='global',
                visible=1,
            ),
            ActionInformation(
                id='mycalendar',
                title='_action_mycalendar_',
                action=Expression(text='string:${portal_url}/Calendars/${member/getUserName}'),
                permissions=('View',),
                condition='member',
                category='user',
                visible=1,
            ),
        ])

    for tool, newactions in actionadd.items():
        actions = list(portal[tool]._actions)
        for action in newactions:
            if action.id not in [ac.id for ac in actions]:
                actions.append(action)
                pr(" Add %s: %s" % (tool, action.id))
        portal[tool]._actions = actions

    # create "Global" folder
    pr("Verifying Global folder")
    ok = 1
    if portalhas('Global'):
        if portal.Global.meta_type != 'Portal Folder':
            pr(" !!! 'Global' is not a Portal Folder")
            primp()
            ok = 0
        else:
            prok()
    else:
        portal.manage_addProduct['CMFCore'].manage_addPortalFolder('Global')
        pr(" Creating 'Global' folder")
    if ok:
        # create global "Preferences"
        pr("Verifying global preferences")
        if 'Preferences' in portal.Global.objectIds():
            prok()
        else:
            pr(" Importing global preferences")
            tryimport(portal.Global, 'Preferences', suffix='zexp', pr=pr)

    # create "Groups" workgroup
    pr("Verifying workgroups")
    if portalhas('Groups'):
        if portal.Groups.meta_type != 'Workgroup':
            pr(" !!! 'Groups' is not a Workgroup")
            primp()
        else:
            prok()
    else:
        pr(" Creating 'Groups' workgroup")
        portal.invokeFactory('Workgroup', 'Groups')
        portal.Groups.setTitle('_str_All_Workgroups_')
        portal.Groups.reindexObject()

    # Help folder XXX

    # 'main' Hierarchy
    pr("Verifying main hierarchy")
    if 'main' in portal.portal_hierarchies.objectIds():
        pr(" Already installed")
    else:
        portal.portal_hierarchies.manage_addHierarchy('main',
                                                      'Reviewed Hierarchy')
        ob = portal.portal_hierarchies.main
        ob.manage_changeProperties(title='_str_Hierarchies_',
                                   root='Public',
                                   skiproot=0)
        pr(" Creating main hierarchy")

    # Public folder
    pr("Verifying 'Public' folder")
    if portalhas('Public'):
        prok()
    else:
        pr(" Creating 'Public' as a hierarchy")
        portal.invokeFactory('Reviewed Hierarchy Level', 'Public')
        Public = portal['Public']
        Public.edit(title="_str_Hierarchies_", level_readers=[], publishable=0)
        Public.rebuildHierarchy()


    if nuxgroupcalendar:
        pr("Verifying 'Calendars' folder")
        if portalhas('Calendars'):
            prok()
        else:
            pr(" Creating 'Calendars'")
            portal.invokeFactory('Calendars', 'Calendars')
            Calendars = portal['Calendars']
            Calendars.setTitle('_str_Calendars_')

    # role/permission mapping
    pr("Verifying permissions")
    permsroot = {
        'Add Workgroups': ['Manager', 'Owner', 'WorkgroupManager'],
        'Add portal content': ['Manager', 'Owner', 'WorkgroupManager', 'WorkgroupMember'],
        'Add portal folders': ['Manager', 'Owner', 'WorkgroupManager', 'WorkgroupMember'],
        'Change permissions': ['Manager', 'Owner', 'WorkgroupManager'],
        'Delete objects': ['Manager', 'Owner', 'WorkgroupManager', 'WorkgroupMember'],
        'Delete replies': ['Manager', 'Owner', 'WorkgroupManager'],
        'FTP access': ['Manager', 'Owner', 'WorkgroupManager', 'WorkgroupMember'], # ?
        'List folder contents': ['Manager', 'Owner', 'WorkgroupManager', 'WorkgroupMember'],
        'Manage Workgroups': ['Manager', 'Owner', 'WorkgroupManager'],
        'Manage properties': ['Manager', 'Owner', 'WorkgroupManager'],
        'Modify portal content': ['Manager', 'Owner', 'WorkgroupManager'],
        'Reply to item': ['Manager', 'Member'],
        'View': ['Manager', 'Owner', 'WorkgroupManager', 'WorkgroupMember', 'WorkgroupVisitor'],
        'View management screens': ['Manager', 'Owner', 'WorkgroupManager', 'WorkgroupMember'],
        'WebDAV Lock items': ['Manager', 'Owner', 'WorkgroupManager', 'WorkgroupMember'],
        'WebDAV Unlock items': ['Manager', 'Owner', 'WorkgroupManager', 'WorkgroupMember'],
        }
    permsgroups = {
        'Access contents information': ('Manager', 'Owner', 'Reader', 'Reviewer', 'WorkgroupManager', 'WorkgroupMember', 'WorkgroupVisitor'),
        'Modify portal content': ('Manager', 'Owner', 'Reviewer', 'WorkgroupManager', 'WorkgroupMember'),
        'View': ('Manager', 'Owner', 'Reader', 'Reviewer', 'WorkgroupManager', 'WorkgroupMember', 'WorkgroupVisitor'),
        }

    pr(" Portal")
    setperms(portal, permsroot, pr=pr)
    portal.reindexObjectSecurity()
    pr(" Workroups")
    setperms(portal.Groups, permsgroups, pr=pr)
    portal.Groups.reindexObjectSecurity()
    pr(" Members")
    setperms(portal.Members, permsgroups, pr=pr)
    portal.Members.reindexObjectSecurity()
    pr(" Reindexing catalog")
    portal.portal_catalog.refreshCatalog(clear=1)

    pr("Verifying private area creation flag")
    if not portal.portal_membership.getMemberareaCreationFlag():
        pr(" Activated")
        portal.portal_membership.setMemberareaCreationFlag()
    else:
        prok()

    pr("Specific upgrades")
    pr("Curent CPS build version in ZODB: %s" % (cps_version, ))
    if this_cps_version == cps_version:
        pr("No need to upgrade")
    else:
        pr("Will upgrade to build version %s" % (this_cps_version, ))
        def upgrade(new_version, portal=portal, pr=pr):
            old_version = portal._cpsversion
            if new_version > old_version:
                pr("Upgrade from %s to %s" % (old_version, new_version))
                portal._cpsversion = new_version
                return 1
            else:
                return 0

        if upgrade(1):
            pr(" Replacing Global/Preferences from Folder to Portal Folder")
            pr("  Creating Preferences_new in Global")
            Global = portal.Global
            Global.invokeFactory('Folder', 'Preferences_new')
            pr("  Creating boxes_container in Preferences_new")
            Global.Preferences_new.invokeFactory('Folder', 'boxes_container')
            new_boxes_container = Global.Preferences_new.boxes_container
            old_boxes_container = Global.Preferences.boxes_container

            box_ids = old_boxes_container.objectIds()
            pr("  Will move boxes: %s" % (box_ids, ))
            cookie = old_boxes_container.manage_cutObjects(box_ids)
            new_boxes_container.manage_pasteObjects(cookie)

            pr("  Remove old Preferences")
            Global._delObject('Preferences')
            pr("  Renaming Preferences_new to Preferences")

            ob = portal.Global.Preferences_new
            Global._delObject('Preferences_new')
            ob = aq_base(ob)
            ob._setId('Preferences')
            Global._setObject('Preferences', ob, set_owner=0)

            pr(" Updating minimizability of global boxes")
            boxes_container = Global.Preferences.boxes_container
            
            styles = portal.boxes_styles_get()
            
            for boxid in box_ids:
                box = getattr(boxes_container, boxid)
                box_style = box.style
                can_minimized = 0
                for style in styles:
                    if style['id'] == box_style:
                        can_minimized = style.get('can_minimized')
                        break
                pr("  Box %s is %sminimizable" % (
                    boxid, ((not can_minimized) and 'not ' or ''),
                ))
                box._can_minimized = can_minimized

            pr(" Cleaning preferences in members' areas")
            Members = portal.Members
            for area in Members.objectValues('Workgroup'):
                pr("  Cleaning %s" % (area.id, ))
                if 'Preferences' in area.objectIds():
                    area._delObject('Preferences')

        if upgrade(2):
            pr(" Upgrading special Workgroups")
            pr("  Upgrading Member's areas")
            Members = portal.Members
            for wg in Members.objectValues('Workgroup'):
                pr("   Upgrading %s" % (wg.id, ))
                wg._computedtitle = 1
            if nuxgroupcalendar:
                pr("  Upgrading Member's calendars")
                Calendars = portal.Calendars
                for cal in Calendars.objectValues('Calendar'):
                    if cal.usertype == 'member':
                        pr("   Upgrading %s" % (cal.id, ))
                        cal._computedtitle = 1

        if upgrade(3):
            pr(" Setting portal_workflow_schema title")
            portal.portal_workflow_schema.setTitle('_wfs_Workflow_Schemas_')

    pr(" Setting site icon")
    portal.icon = '%s/cpssite_icon.png' % (portal.id, )

    # remove cpsinstall external method
    # and fix cpsupdate permission
    if 'cpsinstall' in portal.objectIds():
        pr("Removing cpsinstall")
        portal._delObject('cpsinstall')
    if 'cpsupdate' in portal.objectIds():
        pr("Protecting cpsupdate")
        portal.cpsupdate.manage_permission(
            'View', roles=['Manager'], acquire=0
        )
        portal.cpsupdate.manage_permission(
            'Access contents information', roles=['Manager'], acquire=0
        )

    pr("\nEnd of update")
    return '\n'.join(log)

