# Copyright (c) 2002-2003 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2002 Prfecture du Bas-Rhin, France
# Copyright (c) 2002 CIRB, Belgique
# Authors: Florent Guillaume <mailto:fg@nuxeo.com>
#          Julien Jalon <mailto:jj@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: FAQDocument.py,v 1.11 2003/04/01 21:53:58 fermigier Exp $

"""
  FAQ Document
"""

__version__ = '$Revision: 1.11 $'[11:-2]

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo

from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent

from BaseDocument import BaseDocument, BaseDocument_adder


factory_type_information = (
    {'id': 'FAQ Document',
     'description': 'A FAQ Document is editable through the web.',
     'title': '_portal_type_FAQ',
     'content_icon': 'document_icon.gif',
     'product': 'NuxCPSDocuments',
     'factory': 'addFAQDocument',
     'meta_type': 'FAQ Document',
     'immediate_view': 'faqdocument_edit_form',
     'allow_discussion': 1,
     'filter_content_types': 0,
     'actions': ({'id': 'view',
                  'name': '_action_view_',
                  'action': 'faqdocument_view',
                  'permissions': (View,)},
                 {'id': 'edit',
                  'name': '_action_modify_',
                  'action': 'faqdocument_edit_form',
                  'permissions': (ModifyPortalContent,)},
                 {'id': 'create',
                  'name': '_action_create_',
                  'action': 'faqdocument_create_form',
                  'visible': 0,
                  'permissions': ()},
                 {'id': 'isdocument',
                  'name': 'isdocument',
                  'action': 'isdocument',
                  'visible': 0,
                  'permissions': ()},
                 {'id': 'issearchabledocument',
                  'name': 'issearchabledocument',
                  'action': 'issearchabledocument',
                  'visible': 0,
                  'permissions': ()},
                 ),
     },
    )


class FAQDocument(BaseDocument):
    """FAQ Document"""
    meta_type = 'FAQ Document'

    security = ClassSecurityInfo()

    _properties = BaseDocument._properties + (
        {'id': 'question', 'type': 'text', 'mode': 'w', 'label': 'Question'},
        {'id': 'answer', 'type': 'text', 'mode': 'w', 'label': 'Answer'},
        )
    question = ''
    answer = ''

    #
    # Content accessors
    #
    security.declareProtected(View, 'SearchableText')
    def SearchableText(self):
        """Used by the catalog for basic full text indexing."""
        return '%s %s %s %s' % (self.title, self.description,
                                self.question, self.answer)

InitializeClass(FAQDocument)

def addFAQDocument(dispatcher, id, REQUEST=None, **kw):
    """Add a FAQ Document."""
    ob = FAQDocument(id, **kw)
    return BaseDocument_adder(dispatcher, id, ob, REQUEST=REQUEST)
