# Copyright (c) 2002-2003 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2002 Prfecture du Bas-Rhin, France
# Copyright (c) 2002 CIRB, Belgique
# Authors: Florent Guillaume <mailto:fg@nuxeo.com>
#          Julien Jalon <mailto:jj@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: NewsDocument.py,v 1.12 2003/04/01 21:53:58 fermigier Exp $

"""
  News Document
"""

__version__ = '$Revision: 1.12 $'[11:-2]

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo

from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent

from BaseDocument import BaseDocument, BaseDocument_adder
from cpsdocument_utils import getImageForFile


factory_type_information = (
    {'id': 'News Document',
     'description': 'Une nouvelle est un document court.',
     'title': "_portal_type_News Document",
     'content_icon': 'newsitem_icon.gif',
     'product': 'NuxCPSDocuments',
     'factory': 'addNewsDocument',
     'meta_type': 'News Document',
     'immediate_view': 'newsdocument_edit_form',
     'allow_discussion': 1,
     'filter_content_types': 0,
     'actions': ({'id': 'view',
                  'name': '_action_view_',
                  'action': 'newsdocument_view',
                  'permissions': (View,)},
                 {'id': 'edit',
                  'name': '_action_modify_',
                  'action': 'newsdocument_edit_form',
                  'permissions': (ModifyPortalContent,)},
                 {'id': 'create',
                  'name': '_action_create_',
                  'action': 'newsdocument_create_form',
                  'visible': 0,
                  'permissions': ()},
                 {'id': 'isdocument',
                  'name': 'isdocument',
                  'action': 'isdocument',
                  'visible': 0,
                  'permissions': ()},
                 {'id': 'issearchabledocument',
                  'name': 'issearchabledocument',
                  'action': 'issearchabledocument',
                  'visible': 0,
                  'permissions': ()},
                 ),
     },
    )


class NewsDocument(BaseDocument):
    """News Document"""
    meta_type = 'News Document'

    security = ClassSecurityInfo()

    _properties = BaseDocument._properties + (
        {'id': 'text', 'type': 'text', 'mode': 'w', 'label': 'Text'},
        )
    text = ''

    vignette = None
    photo = None

    def __init__(self, id, vignette='', photo='', **kw):
        BaseDocument.__init__(self, id, **kw)
        self.vignette = getImageForFile(vignette=vignette)
        self.photo = getImageForFile(photo=photo)

    security.declareProtected(ModifyPortalContent, 'edit')
    def edit(self, vignette=None, vignette_change=None, photo=None,
             photo_change=None, **kw):
        """Edit the document."""
        if vignette_change in ('change', None):
            if vignette is not None:
                self.vignette = getImageForFile(vignette=vignette)
        elif vignette_change == 'delete':
            self.vignette = None
        if photo_change in ('change', None):
            if photo is not None:
                self.photo = getImageForFile(photo=photo)
        elif photo_change == 'delete':
            self.photo = None
        BaseDocument.edit(self, **kw)

    #
    # Content accessors
    #
    security.declareProtected(View, 'SearchableText')
    def SearchableText(self):
        """Used by the catalog for basic full text indexing."""
        return '%s %s %s %s' % (self.title, self.description,
                                self.related_links, self.text)

    # XXX: ???
    # def PUT
    # def manage_FTPget
    # def get_size ?

InitializeClass(NewsDocument)


def addNewsDocument(dispatcher, id, REQUEST=None, **kw):
    """Add a News Document."""
    ob = NewsDocument(id, **kw)
    return BaseDocument_adder(dispatcher, id, ob, REQUEST=REQUEST)
