# Copyright (c) 2002-2003 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2002 Prfecture du Bas-Rhin, France
# Copyright (c) 2002 CIRB, Belgique
# Authors: Florent Guillaume <mailto:fg@nuxeo.com>
#          Julien Jalon <mailto:jj@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: TextDocument.py,v 1.12 2003/04/01 21:53:58 fermigier Exp $

"""
  Text Document
"""

__version__ = '$Revision: 1.12 $'[11:-2]

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo

from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent

from BaseDocument import BaseDocument, BaseDocument_adder


factory_type_information = (
    {'id': 'Text Document',
     'description': 'A Text Document is editable through the web.',
     'title': '_portal_type_Text Document',
     'content_icon': 'document_icon.gif',
     'product': 'NuxCPSDocuments',
     'factory': 'addTextDocument',
     'meta_type': 'Text Document',
     'immediate_view': 'textdocument_edit_form',
     'allow_discussion': 1,
     'filter_content_types': 0,
     'actions': ({'id': 'view',
                  'name': '_action_view_',
                  'action': 'textdocument_view',
                  'permissions': (View,)},
                 {'id': 'edit',
                  'name': '_action_modify_',
                  'action': 'textdocument_edit_form',
                  'permissions': (ModifyPortalContent,)},
                 {'id': 'create',
                  'name': '_action_create_',
                  'action': 'textdocument_create_form',
                  'visible': 0,
                  'permissions': ()},
                 {'id': 'isdocument',
                  'name': 'isdocument',
                  'action': 'isdocument',
                  'visible': 0,
                  'permissions': ()},
                 {'id': 'issearchabledocument',
                  'name': 'issearchabledocument',
                  'action': 'issearchabledocument',
                  'visible': 0,
                  'permissions': ()},
                 ),
     },
    )


class TextDocument(BaseDocument):
    """Text Document"""
    meta_type = 'Text Document'

    security = ClassSecurityInfo()

    _properties = BaseDocument._properties + (
        {'id': 'text', 'type': 'text', 'mode': 'w', 'label': 'Text'},
        )
    text = ''

    #
    # Content accessors
    #
    security.declareProtected(View, 'SearchableText')
    def SearchableText(self):
        """Used by the catalog for basic full text indexing."""
        return '%s %s %s %s' % (self.title, self.description,
                                self.related_links, self.text)

    # XXX: what's this ?
    # def PUT
    # def manage_FTPget
    # def get_size ?


InitializeClass(TextDocument)


def addTextDocument(dispatcher, id, REQUEST=None, **kw):
    """Add a Text Document."""
    ob = TextDocument(id, **kw)
    return BaseDocument_adder(dispatcher, id, ob, REQUEST=REQUEST)

