# (c) 2002 Nuxeo SARL <http://nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: ForumNotificationTool.py,v 1.9 2003/01/16 14:35:21 smigniot Exp $
"""ForumNotificationTool

"""

__version__='$Revision: 1.9 $'[11:-2]

from zLOG import LOG, INFO

from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo

from OFS.SimpleItem import SimpleItem

from Products.CMFCore.CMFCorePermissions import View, ManagePortal
from Products.CMFCore.utils import UniqueObject, getToolByName
from util import log

class ForumNotificationTool(UniqueObject, SimpleItem):
    """Forum Notification Tool.
    
    A central (unique for each CPS) registry of forum notifications.
    """

    id = 'portal_forum_notification'
    meta_type = 'Forum Notification Tool'
    security = ClassSecurityInfo()

    security.declareProtected('__init__')
    def __init__(self):
        """ Creates an empty notification tool """
        self._subscriptions = ()


    security.declareProtected(View, 'subscribe')
    def subscribe(self, member, object):
        """ Subscribes a member to the service """
        object_link = { "path": "/".join(object.getPhysicalPath()),
                        "meta_type": object.meta_type}
        if not self.isSubscribed(member, object):
            self._subscriptions += ((member.getId(), object_link),)

    security.declareProtected(View, 'isSubscribed')
    def isSubscribed(self, member, object):
        """Return true if user is subscribed to object."""
        member_id = member.getId()
        object_path = "/".join(object.getPhysicalPath())
        for (user, link) in self._subscriptions:
            if member_id == user and link["path"] == object_path:
                return 1
        return 0

    security.declareProtected(View, 'unsubscribe')
    def unsubscribe(self, member, object=None):
        """Unsubscribe the member to object, or remove all subscriptions 
        if object == None."""
        member_id = member.getId()
        if object == None:
            l = [ (user, link) for (user, link) 
                    in self._subscriptions if member_id != user ]
        else:
            l = []
            object_path = "/".join(object.getPhysicalPath())
            for (user, link) in self._subscriptions:
                if member_id != user or link["path"] != object_path:
                    l.append((user, link))
        self._subscriptions = tuple(l)

    security.declareProtected(View, 'unsubscribeAll')
    def unsubscribeAll(self, member, paths):
        for path in paths:
            #log("unsubscribing path = "+path)
            object = self.unrestrictedTraverse(path)
            self.unsubscribe(member, object)

    security.declareProtected(View, 'listSubscribersFor')
    def listSubscribersFor(self, object):
        """Return list of member for object."""
        object_path = "/".join(object.getPhysicalPath())
        d = {}
        for (user, link) in self._subscriptions:
            # log("subscriber "+str((user,link),))
            if object_path.startswith(link["path"]):
                d[user] = None
        # log("subscriptions to %s = %s" % (object_path, str(d.keys())))
        return [ user for user in d.keys() ]

    security.declareProtected(View, 'listObservedObjectsFor')
    def listObservedObjectsFor(self, member):
        """Return list of objects observed by member."""
        member_id = member.getId()
        l = []
        for (user, link) in self._subscriptions:
            if user == member_id:
                object = self.unrestrictedTraverse(link["path"], None)
                if object:
                    l.append(object)

        return l

    def debug(self, message):
        log("NotificationTool: "+message)


InitializeClass(ForumNotificationTool)
