#
# (C) Copyright 2002 Nuxeo SARL <http://www.nuxeo.com/>
# This product is governed by a license. See LICENSE.txt file.
#

# Zope
from zLOG import LOG, DEBUG
from Globals       import InitializeClass
from AccessControl import ClassSecurityInfo

# CMF
from Products.CMFCore.CMFCorePermissions \
    import View, ManageProperties, setDefaultRoles
    
# CPS
from Products.NuxCPSDocuments.BaseDocument import BaseDocument

from Forum import ManageForumContent

setDefaultRoles(ManageForumContent, ('Manager', 'Owner', 'WorkgroupManager'))

factory_type_information = ( { 'id'             : 'ThreadSnapShot'
                             , 'meta_type'      : 'ThreadSnapShot'
                             , 'description'    : """ ThreadSnapShot \
                             hold threaded discussions."""
                             , 'icon'           : 'ThreadSnapShot.gif'
                             , 'product'        : 'NuxCPSForum'
                             , 'factory'        : 'add_snapshot'
                             , 'filter_content_types' : 0
                             , 'allowed_content_types' : () 
                             , 'immediate_view' : 'folder_edit_form'
                             , 'actions'        :
                                ( { 'id'            : 'view'
                                  , 'name'          : '_action_view_'
                                  , 'action'        : 'thread_snapshot_view'
                                  , 'permissions'   : (View,)
                                  }
                                , { 'id'            : 'edit'
                                  , 'name'          : '_action_modify_'
                                  , 'action'        : 'thread_snapshot_edit_form'
                                  , 'permissions'   : (ManageForumContent, )
                                  }
                                , {'id': 'issearchabledocument',
                                  'name': 'issearchabledocument',
                                  'action': 'issearchabledocument',
                                  'visible': 0,
                                  'permissions': ()},
                                
                                )
                             }
                           ,
                           )

class ThreadSnapShot(BaseDocument): 
    """This class clones a discussion thread.

    The goal is here to be able to publish a discussion.
    It makes a new object with the content of a thread.
    It becomes a static object.
    After initialization, it's possible to edit the document metadata.
    """

    meta_type = "ThreadSnapShot"
    
    security = ClassSecurityInfo()

    security.declareProtected(ManageForumContent, "setContent")
    def setContent(self, content, children_content):
        """Set the HTML thread view content."""
        self.body_content = content
        self.children_content = children_content

    security.declarePublic("getContent")
    def getContent(self):
        """Return the message root thread content."""
        return self.body_content
    
    security.declarePublic("getChildrenContent")
    def getChildrenContent(self):
        """Return the children linear list."""
        return self.children_content

    security.declareProtected(ManageForumContent, "SearchableText")
    def SearchableText(self):
        """Indexing the Thread content."""
        s = '%s %s' % (self.Title(), self.Description())
        #
        # The root message
        #
        for item in self.getContent():
            s += ' %s %s' % (item['title'], item['text'])

        #
        # Children messages
        #
        for item in self.getChildrenContent():
            s += ' %s %s' % (item['title'], item['text'])

        return s
        
    
InitializeClass(ThreadSnapShot)


#
#  Add ThreadSnapShot Object
#
def _take(child, content, depth):
    """Take one child."""
    content.append({'depth': depth,
                    'title'   : child.title,
                    'author'  : child.author,
                    'date'    : child.created,
                    'content' : child.CookedBody(),
                    'text'    : child.text,
                    })
    return child.children()


def _loop(children, content, depth):
    """Main loop over children."""
    for child in children:
        new_children = _take(child, content, depth)
        if new_children!= []:
            depth = depth + 1
            _loop(new_children, content, depth) 


def add_snapshot(self, id, title, original_thread):
    """Add an instance of this."""
    tss = ThreadSnapShot(id, title=title)
    content = []
    #
    # Setting the content list 
    #
    
    # root message
    content.append({'depth': 0,
                    'title'   : original_thread.title,
                    'author'  : original_thread.author,
                    'date'    : original_thread.created,
                    'content' : original_thread.CookedBody(),
                    'text'    : original_thread.text,
                    })
     
    children = original_thread.children()
    LOG('NCF', DEBUG, 'Children: %s' % (children, ))
    children_content = []
    
    _loop(children, children_content, 1)
    
    tss.setContent(content, children_content)
    self._setObject(id, tss)

