# Copyright (c) 2002 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2002 Prfecture du Bas-Rhin, France
# Author: Florent Guillaume <mailto:fg@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: HierarchiesTool.py,v 1.5 2003/07/14 21:30:14 fermigier Exp $

"""
  Hierarchies Tool
"""

__version__='$Revision: 1.5 $'[11:-2]

import string
from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo

from OFS.Folder import Folder

from Products.CMFCore.CMFCorePermissions import setDefaultRoles, \
     AccessContentsInformation, ManagePortal
from Products.CMFCore.utils import UniqueObject


RebuildHierarchy = 'Rebuild Hierarchy'
setDefaultRoles(RebuildHierarchy, ('Manager',))

AddHierarchyLevel = 'Add Hierarchy Level'
setDefaultRoles(AddHierarchyLevel, ('Manager',))

ChangeHierarchyLevel = 'Change Hierarchy Level'
setDefaultRoles(ChangeHierarchyLevel, ('Manager',))


def addHierarchiesTool(container, id, REQUEST=None):
    """Add a Hierarchies Tool."""
    obj = HierarchiesTool(id)
    container._setObject(id, obj)
    if REQUEST is not None:
        REQUEST.RESPONSE.redirect(container.absolute_url() + '/manage_main')


class HierarchiesTool(UniqueObject, Folder):
    """
    Hierarchies Tool.
    """
    id = 'portal_hierarchies'
    meta_type = 'Hierarchies Tool'

    security = ClassSecurityInfo()

    manage_options = Folder.manage_options

    #
    # ZMI methods
    #

    def all_meta_types(self):
        all = ({'name': 'Hierarchy',
                'action': 'manage_addHierarchyForm',
                'permission': ManagePortal},
               )
        return all

    _manage_addHierarchyForm = DTMLFile('dtml/manage_addHierarchyForm', 
                                        globals())

    security.declareProtected(ManagePortal, 'manage_addHierarchyForm')
    def manage_addHierarchyForm(self, REQUEST):
        """Form to add a new Hierarchy."""
        types = _hierarchy_factories.keys()
        types.sort()
        return self._manage_addHierarchyForm(REQUEST, types=types)

    security.declareProtected(ManagePortal, 'manage_addHierarchy')
    def manage_addHierarchy(self, id, hier_type, REQUEST=None):
        """Add a new Hierarchy."""
        container = self
        factory = _hierarchy_factories[hier_type]
        obj = factory(id)
        container._setObject(id, obj)
        obj = container._getOb(id)
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(obj.absolute_url() + '/manage_main')

    #
    # Public methods
    #

    security.declarePublic('getHierarchyById')
    def getHierarchyById(self, id):
        """Get a Hierarchy object by id."""
        hier = self._getOb(id, None)
        if hier is not None:
            return hier
        else:
            raise AttributeError, ('No such hierarchy "%s"' % id)

    security.declarePublic('getHierarchies')
    def getHierarchies(self):
        """Get all hierarchy ids."""
        return self.objectIds()

    security.declareProtected(AccessContentsInformation, 'getLevelByKey')
    def getLevelByKey(self, levelkey):
        """Find the level object with a given key in all hierarchies."""
        hiers = self.getHierarchies()
        for hier in hiers:
            level = getattr(self, hier).getLevelByKey(levelkey)
            if level is not None:
                return level
        return None


InitializeClass(HierarchiesTool)


_hierarchy_factories = {}

def addHierarchyFactory(factory):
    key = factory.meta_type
    _hierarchy_factories[key] = factory

