# Copyright (c) 2002 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2002 Prfecture du Bas-Rhin, France
# Author: Florent Guillaume <mailto:fg@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: SimpleHierarchyLevel.py,v 1.8 2003/07/14 21:30:14 fermigier Exp $

"""
  Simple Hierarchy Level
"""

__version__ = '$Revision: 1.8 $'[11:-2]

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo

from Products.CMFCore.CMFCorePermissions import View

from HierarchiesTool import ChangeHierarchyLevel
from BaseHierarchyLevel import BaseHierarchyLevel


factory_type_information = (
    {'id': 'Simple Hierarchy Level',
     'description': ('A Simple Hierarchy Level belongs to a Hierarchy and '
                     'has a corresponding query into the catalog.'),
     'meta_type': 'Simple Hierarchy Level',
     'content_icon': 'hierarchylevel_icon.gif',
     'product': 'NuxHierarchies',
     'factory': 'addSimpleHierarchyLevel',
     'immediate_view': 'simplehierarchylevel_edit_form',
     'filter_content_types': 0,
     'actions': ({'id': 'view',
                  'name': 'View',
                  'action': 'simplehierarchylevel_view',
                  'permissions': (View,)},
                 {'id': 'edit',
                  'name': 'Edit',
                  'action': 'simplehierarchylevel_edit_form',
                  'permissions': (ChangeHierarchyLevel,)},
                 ),
     },
    )


class SimpleHierarchyLevel(BaseHierarchyLevel):
    """Simple Hierarchy Level

    This is a Skinned Folder which belongs to a Hierarchy and has a
    corresponding key.
    """
    meta_type = 'Simple Hierarchy Level'

    _properties = (
        {'id': 'title', 'type': 'string', 'mode': 'w', 
            'label': 'title'},
        {'id': 'description', 'type': 'text', 'mode': 'w', 
            'label': 'description'},
        {'id': 'hierarchyid', 'type':'string', 'mode': 'w', 
            'label': 'Hierarchy name'},
        {'id': 'levelkey', 'type':'string', 'mode': 'w', 
            'label': 'Level key'},
        {'id': 'publishable', 'type': 'boolean', 'mode': 'w', 
            'label': 'Publication allowed'},
        )

    title = ''
    description = ''
    hierarchyid = 'main'
    levelkey = ''
    publishable = 1

    security = ClassSecurityInfo()

    def __init__(self, id,
                 title='',
                 description='',
                 hierarchyid='main',
                 levelkey='',
                 publishable=1):
        self.id = id
        self.title = title
        self.description = description
        self.hierarchyid = hierarchyid
        self.levelkey = levelkey
        self.publishable = publishable

    def edit(self,
             title=None,
             description=None,
             hierarchyid=None,
             levelkey=None,
             publishable=None,
             REQUEST=None):
        """Edit a Simple Hierarchy Level."""
        if title is not None: 
            self.title = title
        if description is not None: 
            self.description = description
        if levelkey is not None: 
            self.levelkey = levelkey
        if hierarchyid is not None: 
            self.hierarchyid = hierarchyid
        if publishable is not None: 
            self.publishable = publishable

        if REQUEST is not None:
            REQUEST.RESPONSE.redirect('%s/simplehierarchylevel_view' 
                % self.absolute_url())


InitializeClass(SimpleHierarchyLevel)


def addSimpleHierarchyLevel(dispatcher, id, REQUEST=None):
    """Add a Simple Hierarchy Level."""
    obj = SimpleHierarchyLevel(id)
    container = dispatcher.Destination()
    container._setObject(id, obj)
    obj = container._getOb(id)
    # set default hierarchyid to its parent's
    if hasattr(container, 'getProperty'):
        hierarchyid = container.getProperty('hierarchyid', None)
        if hierarchyid is not None and type(hierarchyid) is type(''):
            obj.hierarchyid = hierarchyid
    if REQUEST is not None:
        url = dispatcher.DestinationURL()
        REQUEST.RESPONSE.redirect('%s/manage_main' % url)

