import time, calendar, string, types

YEAR,MONTH,DAY,HOUR,MINUTE,SECOND,WEEKDAY,JULIANDAY,DAYLIGHT = range(9)
VALIDMINS = range(0,60)
VALIDHOURS = range(0,24)
VALIDDOMS = range(1,32)
VALIDMONTHS = range(1,13)
VALIDDOWS = range(0,7)
DOWS = calendar.day_abbr
LOWER_DOWS = map(string.lower, DOWS)
MONTHS = calendar.month_abbr
PERIODS = ['minute','hour','day','week','month','year']

def processInt(val, valid, desc):
    if val in valid:
        return [val]
    raise ValueError, 'Invalid %s integer %s' % (desc, val)

def processString(val, valid, desc):
    sets = filter(None, map(string.strip, val.split(',')))
    ranges = []
    for set in sets:
        is_range = set.count('-')
        if is_range:
            if is_range > 1:
                raise ValueError, 'Too many dashes in range value %s' % set
            try:
                begin, end = map(int, set.split('-'))
                if begin > end:  begin, end = end, begin
                ranges.append([begin, end])
            except ValueError:
                raise ValueError, 'Invalid value in range %s' % set
        else:
            try:
                val = int(set)
                ranges.append([val, val])
            except ValueError:
                raise ValueError, 'invalid value %s' % set
    l = []
    for range in ranges:
        l = l + range(range[0], range[1] + 1)
    return processList(l, valid, err, desc)

def processList(val, valid, desc):
    l = dedupe(val)
    l = sort(l)
    try:
        l = intify(l)
    except ValueError:
        msg = '%s list has a member unconvertable to an int: %s' % (desc, l)
        raise ValueError, msg
               
    bad = map(lambda x, valid=valid: x not in valid, l)
    if 1 in bad:
        raise ValueError, '%s list has an invalid value: %s' % (desc, l)
    return l

def dedupe(seq):
    d = {}
    for item in seq:
        d[item] = 1
    return d.keys()

def sort(seq):
    if isinstance(seq, types.ListType):
        l = seq
    else:
        l = list(seq)
    l.sort()
    return l

def intify(seq):
    return map(int, seq)

def binary_max(seq, t):
    slen = len(seq)
    min = 0; max = slen - 1
    while 1:
        if max < min:
            # linear search from the center finds the first value greater
            # than t
            for index in range(m,slen):
                val = seq[index]
                if val > t:
                    return val
            else:
                # if we wrapped around, return the first element
                return seq[0]
        m = (min + max) / 2
        if seq[m] < t:
            min = m + 1
        elif seq[m] > t:
            max = m - 1
        else:
            return seq[m]

def get_weekdays(days, month_start_weekday, last_month_day):
    if filter(None, map(lambda x: 0 > x > 6, days)):
        raise ValueError, 'improper days value %s' % days
    l = []
    if 0 > month_start_weekday > 6:
        raise ValueError, ('improper month start weekday %s' %
                           month_start_weekday)
    if 0 > last_month_day > 31:
        raise ValueError, ('improper last_month_day %s' %
                           last_month_day)
    i = month_start_weekday
    for x in range(1, last_month_day+1):
        if i in days:
            l.append(x)
        i = i + 1
        if i > 6:
            i = 0
    return l

class Schedule:
    def __init__(self, minutes=None, hours=None, days=None, months=None,
                 weekdays=None):
        self.reset(minutes, hours, days, months, weekdays)

    def reset(self, minutes=None, hours=None, days=None, months=None,
              weekdays=None):
        self.minutes = self.getRange(minutes, VALIDMINS, desc='Minute')
        self.hours = self.getRange(hours, VALIDHOURS, desc='Hour')
        self.days = self.getRange(days, VALIDDOMS, desc='Day of month')
        self.months = self.getRange(months, VALIDMONTHS, desc='Month')
        self.weekdays = self.getRange(weekdays, VALIDDOWS, desc='Day of week')
        if self.days != VALIDDOMS and self.weekdays != VALIDDOWS:
            raise ValueError, ('Cannot specify both days of week and '
                               'days of month')
        if self.days != VALIDDOMS:
            self.daymode = 'days'
        else:
            self.daymode = 'days_of_week'

    dispatch = {
        types.NoneType:   None,
        types.IntType:    processInt,
        types.FloatType:  processInt,
        types.ListType:   processList,
        types.TupleType:  processList,
        types.StringType: processString,
        }

    def getRange(self, val, valid, desc):
        f = self.dispatch.get(type(val), None)
        if f is None:
            return valid
        else:
            return apply(f, (val, valid, desc))

    def getPrintableNext(self, when=None):
        return time.ctime(self.getEpochNext(when))

    def getEpochNext(self, when=None):
        return time.mktime(self.getTupleNext(when))

    getNext = getEpochNext

    def getTupleNext(self, when=None):
        if when is None:
            when = time.time()
        t = time.localtime(when)
        year = t[YEAR]
        month = t[MONTH]
        day = t[DAY]
        hour = t[HOUR]
        minute = t[MINUTE]
        weekday = t[WEEKDAY]
        second = 0    # doesn't matter
        julianday = 0 # doesn't matter
        dst = -1      # try to automatically do the right thing
        month_start_weekday, last_month_day=calendar.monthrange(year, month)
        next_minute = binary_max(self.minutes, minute)
        if next_minute < minute:
            hour = hour + 1
        next_hour = binary_max(self.hours, hour)
        if next_hour < hour:
            day = day + 1
        if self.daymode == 'days':
            real_days = self.days
            if last_month_day < 31:
                real_days=filter(lambda x, last=last_month_day:last>=x,
                                 self.days)
            next_day = binary_max(real_days, day)
            if next_day < day:
                month = month + 1
        if self.daymode == 'days_of_week':
            real_days = get_weekdays(self.weekdays, month_start_weekday,
                                     last_month_day)
            next_day = binary_max(real_days, day)
            if next_day < day:
                # check next month
                day = 1
                month = month + 1
                if month > 12:
                    month = 0
                    year = year + 1
                msw, lmd = calendar.monthrange(year, month)
                real_days = get_weekdays(self.weekdays, msw, lmd)
                next_day = binary_max(real_days, day)
        next_month = binary_max(self.months, month)
        if next_month < month:
            next_year = year + 1
        else:
            next_year = year
        return (next_year, next_month, next_day, next_hour, next_minute,
                second, weekday, julianday, dst)
    
        
    
