# (C) Copyright 2002 Nuxeo SARL <http://nuxeo.com>
# (C) Copyright 2002 Florent Guillaume <mailto:fg@nuxeo.com>
# (C) Copyright 2002 Prfecture du Bas-Rhin, France
# (C) Copyright 2002 Adelux <http://adequat.c2a.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: WorkflowSchema.py,v 1.22 2003/07/15 13:26:54 guillaum Exp $

from zLOG import LOG, DEBUG, PROBLEM

import os, urllib, string, random

from Globals import InitializeClass, DTMLFile
from DocumentTemplate.DocumentTemplate import HTML
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Acquisition import aq_base, aq_inner, aq_parent
from MiscTools import legal_id, put_script

from OFS.Folder import Folder
from OFS.PropertyManager import PropertyManager

from Products.CMFCore.utils import _getViewFor
from Products.CMFCore.CMFCorePermissions import View
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.NuxUserGroups.CatalogToolWithGroups import mergedLocalRoles

from WorkflowSchemaPermissions import AddWorkflowSchema


mail_send_template = """\
From: %s
To: %s
Subject: %s
Content-Type: text/plain; charset=ISO-8859-1
Mime-Version: 1.0

%s"""



Schema_Type = 'Workflow Schema'

factory_type_information = (
    {'id': Schema_Type,
     'meta_type': Schema_Type,
     'description': '',
     'content_icon': 'WFS_icon.gif',
     'product': 'WorkflowSchema',
     'factory': 'addWorkflowSchema',
     'immediate_view': 'wfschema_schema_view',
     'filter_content_types': 1,
     'allowed_content_types': ('Workflow Schema State',),
     'actions': ({'id': 'view',
                  'name': 'Voir',
                  'action': 'wfschema_schema_view',
                  'permissions': (View,)},
                 ),
     },
    {'id': 'Workflow Schema State',
     'meta_type': 'Workflow Schema State',
     'description': '',
     'content_icon': 'folder_icon.gif',
     'product': 'WorkflowSchema',
     'factory': 'addState',
     'immediate_view': 'wfschema_state_view',
     'filter_content_types': 1,
     'allowed_content_types': ('Workflow Schema Event Container',
                               'Workflow Schema Condition Container',),
     'actions': ({'id': 'view',
                  'name': '_action_view_',
                  'action': 'wfschema_state_view',
                  'permissions': (View,)},
                 ),
     },
    {'id': 'Workflow Schema Event Container',
     'meta_type': 'Workflow Schema Event Container',
     'description': '',
     'content_icon':'folder_icon.gif',
     'product': 'WorkflowSchema',
     'factory': 'addEventContainer',
     'immediate_view': '',
     'filter_content_types': 1,
     'allowed_content_types': ('Workflow Schema Mail Event',
                               'Workflow Schema Publication Switch Event',),
     'actions': ({'id': 'view',
                  'name': '_action_view_',
                  'action': 'wfschema_container_view',
                  'permissions': (View,)},
                 ),
     },
    {'id': 'Workflow Schema Mail Event',
     'meta_type': 'Workflow Schema Mail Event',
     'description': '',
     'content_icon':'document_icon.gif',
     'product': 'WorkflowSchema',
     'factory': 'addMailEvent',
     'immediate_view': '',
     'filter_content_types': 1,
     'allowed_content_types': (),
     'actions': ({'id': 'edit',
                  'name': '_action_modify_',
                  'action': 'wfschema_mail_event_edit_form',
                  'permissions': (AddWorkflowSchema,)},
                 ),
     },
    {'id': 'Workflow Schema Publication Switch Event',
     'meta_type': 'Workflow Schema Publication Switch Event',
     'description': '',
     'content_icon':'document_icon.gif',
     'product': 'WorkflowSchema',
     'factory': 'addPublicationSwitchEvent',
     'immediate_view': '',
     'filter_content_types': 1,
     'allowed_content_types': (),
     'actions': ({'id': 'edit',
                  'name': '_action_modify_',
                  'action': 'wfschema_publication_switch_event_edit_form',
                  'permissions': (AddWorkflowSchema,)},
                 ),
     },
    {'id': 'Workflow Schema Condition Container',
     'meta_type': 'Workflow Schema Condition Container',
     'description': '',
     'content_icon':'folder_icon.gif',
     'product': 'WorkflowSchema',
     'factory': 'addConditionContainer',
     'immediate_view': '',
     'filter_content_types': 1,
     'allowed_content_types': ('Workflow Schema Substate Condition',
                               'Workflow Schema Signature Condition',),
     'actions': ({'id': 'view',
                  'name': '_action_view_',
                  'action': 'wfschema_container_view',
                  'permissions': (View,)},
                 ),
     },
    {'id': 'Workflow Schema Substate Condition',
     'meta_type': 'Workflow Schema Substate Condition',
     'description': '',
     'content_icon':'document_icon.gif',
     'product': 'WorkflowSchema',
     'factory': 'addSubstateCondition',
     'immediate_view': '',
     'filter_content_types': 1,
     'allowed_content_types': (),
     'actions': ({'id': 'edit',
                  'name': '_action_modify_',
                  'action': 'wfschema_substate_condition_edit_form',
                  'permissions': (AddWorkflowSchema,)},
                 ),
     },
    {'id': 'Workflow Schema Signature Condition',
     'meta_type': 'Workflow Schema Signature Condition',
     'description': '',
     'content_icon':'document_icon.gif',
     'product': 'WorkflowSchema',
     'factory': 'addSignatureCondition',
     'immediate_view': '',
     'filter_content_types': 1,
     'allowed_content_types': (),
     'actions': ({'id': 'edit',
                  'name': '_action_modify_',
                  'action': 'wfschema_signature_condition_edit_form',
                  'permissions': (AddWorkflowSchema,)},
                 #{'id': 'signatures',
                 # 'name': '_action_signatures_',
                 # 'action': 'wfschema_signature_condition_signatures_form',
                 # 'permissions': (AddWorkflowSchema,)},
                 ),
     },
    )


def addWorkflowSchema(container, id,
                      REQUEST=None, **kw):
    """Create an empty schema, assign an id_in_tool that will be unique in workflow_tool"""
    if REQUEST is not None:
        kw.update(REQUEST.form)
    if kw.has_key('id'): del kw['id']
    wf_tool = getToolByName(container, 'portal_workflow')
    if not id.startswith('schema_'):
        id = 'schema_'+id
    id = legal_id(id)

    #XXX find out why it will not create the object through prtal_types?
    #(this will help with action box)
    #pt = getToolByName( container, 'portal_types' )
    #pt.constructContent('WorkflowSchema', container, id)
    #container.invokeFactory(id=id, type_name='WorkflowSchema', **kw)

    ob = WorkflowSchema(id, **kw)
    container._setObject(id, ob)
    schema = container._getOb(id)

    # Add final publication state
    sid = '00'
    addState(schema, sid, title="Publication")
    state = getattr(schema, sid)
    eid = state.incoming_events.manage_addPublicationSwitchEvent(
        allow=1, leave_schema=0)
    state.incoming_events[eid].title = 'Publication'
    eid = state.outgoing_events.manage_addPublicationSwitchEvent(
        allow=0, leave_schema=0)
    state.outgoing_events[eid].title = 'Stop Publication'
    # needed by CMF ConstructInstance because id may have changed
    return id


class WorkflowSchema(PortalFolder, PortalContent, DefaultDublinCoreImpl):
    """
    A workflow schema serves as a proxy for the CMF user to be able to
    crate new Workflow definitions without direct contact with the
    workflow tool.
    """

    meta_type = Schema_Type
    portal_type = Schema_Type

    #def view(self,REQUEST):
    #    """ """
    #    return self.schema_view(self)


    _properties = (
        {'id':'title', 'type':'string', 'mode':'w', 'label':'Title'},
        {'id':'description', 'type':'text', 'mode':'w', 'label':'Description'},
        {'id':'associated_portal_types', 'type':'lines', 'mode':'w',
         'label':'Associated Portal Types'},
        )
    title = ''
    description = ''
    associated_portal_types = []

    ##when this attribute is true CMF points to folder_contents, so we set it to false
    #isPrincipiaFolderish = 0

    # Declarative security
    security = ClassSecurityInfo()
    security.declareObjectProtected(View)

    def __init__(self, id,
                 title='',
                 description='',
                 associated_portal_types=[],
                 **kw
                 ):
        self.id = id
        self.title = title
        self.description = description
        self.associated_portal_types = associated_portal_types
        #self.associated_objects = []

    def getIdInTool(self):
        return self.getId()

    def manage_afterAdd(self, item, container):
        if aq_base(self) is not aq_base(container):
            self.generate_workflow()
        WorkflowSchema.inheritedAttribute('manage_afterAdd')(self, item, container)

    def manage_beforeDelete(self, item, container):
        WorkflowSchema.inheritedAttribute('manage_beforeDelete')(self, item, container)
        if aq_base(self) is not aq_base(container):
            self.delete_workflow()

    #
    # ZMI
    #

    #manage_options = (
    #    ({'label':'States', 'action':'manage_main'},) +
    #    PortalFolder.manage_options[1:] # Properties
    #    )

    all_meta_types = (
        {'name': 'Workflow Schema State',
         'action': 'manage_addStateForm',
         'permission': AddWorkflowSchema,
         },
        )

    manage_addStateForm = DTMLFile('zmi/addStateForm', globals())

    security.declareProtected(AddWorkflowSchema, 'manage_addState')
    def manage_addState(self, id, REQUEST=None, **kw):
        """Adds a State from ZMI."""
        if REQUEST is not None:
            kw.update(REQUEST.form)
        if kw.has_key('id'):
            del kw['id']
        addState(self, id, REQUEST=REQUEST, **kw)

    #
    # Used by PropertyManager
    #

    security.declarePublic('propertyLabel')
    def propertyLabel(self, id):
        """Return a label for the given property id."""
        for p in self._properties:
            if p['id'] == id:
                return p.get('label', id)
        return id

    #
    # Implementation
    #

    security.declareProtected(View, 'getWorkflowSchema')
    def getWorkflowSchema(self):
        return self

    security.declareProtected(View, 'getWorkflow')
    def getWorkflow(self):
        workflow_tool = getToolByName(self, 'portal_workflow')
        wf = getattr(workflow_tool, self.getIdInTool()) # , None
        #if wf is None:
        #    self.generate_workflow()
        #    wf = getattr(workflow_tool, self.getIdInTool())
        return wf

    security.declareProtected(View, 'getSchemaStateIds')
    def getSchemaStateIds(self, onlywf=1):
        """
        Gets the schema state ids, sorted in order.
        """
        mt = State.meta_type
        wfs_ids = self.objectIds(mt)
        wfs_ids.sort() # sort by id XXX change to an ordered folder
        return wfs_ids

    # unused
    def getSchemaStates(self, onlywf=1):
        """
        Gets the schema state ids, sorted in order.
        """
        mt = State.meta_type
        return self.objectValues(mt) #unsorted: change to ordered folder

    security.declareProtected(View, 'getWorkflowStateIds')
    def getWorkflowStateIds(self, onlywf=1):
        wfs_ids = self.getSchemaStateIds()
        ids = self.getWorkflow().states.objectIds()
        wf_ids = [id for id in wfs_ids if id in ids] # keep order
        return wf_ids

    # Called by workflow script during transition between wf states.
    # XXX security ?
    security.declareProtected(View, 'execute_transition')
    def execute_transition(self, state_change):
        LOG('execute_transition', DEBUG, '=> %s (%s)' % (state_change.new_state, state_change.transition.getId()))
        object = state_change.object
        wf_old_state = state_change.old_state
        wf_new_state = state_change.new_state
        if wf_old_state == wf_new_state: # initial transition at creation XXX ???
            LOG('WorkflowSchema', DEBUG, 'old_state=new_state=%s' % wf_old_state.getId())
            wf_old_state = None
        # do outgoing events
        if wf_old_state is not None:
            LOG('WorkflowSchema', DEBUG, 'execute_transition from %s' % wf_old_state.getId())
            state_id = wf_old_state.getId()
            state = getattr(self, state_id)
            container = state.outgoing_events
            for ob in container.objectValues():
                ob.execute_event(object)
        # do incoming events (and cleanup)
        if wf_new_state is not None:
            LOG('WorkflowSchema', DEBUG, 'execute_transition to %s' % wf_new_state.getId())
            state_id = wf_new_state.getId()
            state = getattr(self, state_id)
            container = state.incoming_events
            for ob in container.objectValues():
                ob.execute_event(object)
            container = state.outgoing_conditions
            for ob in container.objectValues():
                ob.cleanup(object) # XXX or when leaving old state ?

    # Called by workflow script during loop transition.
    # A loop transition keeps in the same wf state but changes
    # some internal variables.
    # XXX security ?
    security.declareProtected(View, 'execute_loop_transition')
    def execute_loop_transition(self, state_change):
        LOG('execute_loop_transition', DEBUG, '=> %s (%s)' % (state_change.new_state, state_change.transition.getId()))
        object = state_change.object
        new_state = state_change.new_state
        transition_id = state_change.transition.getId()
        if new_state is not None:
            state_id = new_state.getId()
            container = getattr(self, state_id).outgoing_conditions
            for cond in container.objectValues():
                if cond.get_transition_id() == transition_id:
                    cond.execute_event(object)
                    break

    # Verify a transition guard between states.
    # All existing outgoing_conditions must be met.
    # called by workflow guard
    security.declareProtected(View, 'verify_guard')
    def verify_guard(self, object, state_id, transition_id):
        LOG('verify_guard', DEBUG, 'state=%s transition=%s'
            % (state_id, transition_id))
        container = getattr(self, state_id).outgoing_conditions
        for cond in container.objectValues():
            ok = cond.verify_guard(object)
            if not ok:
                LOG('verify_guard', DEBUG, 'returning 0')
                return 0
        LOG('verify_guard', DEBUG, 'returning 1')
        return 1

    # Verify a backward transition guard between states.
    # called by workflow guard
    security.declareProtected(View, 'verify_back_guard')
    def verify_back_guard(self, object, state_id, transition_id):
        LOG('verify_back_guard', DEBUG, 'state=%s transition=%s'
            % (state_id, transition_id))
        if self.REQUEST['AUTHENTICATED_USER'].has_role('Manager'):
            LOG('verify_back_guard', DEBUG, 'returning 1, Manager')
            return 1
        container = getattr(self, state_id).backward_conditions
        for cond in container.objectValues():
            # Only look at signature conditions
            if cond.meta_type != SignatureCondition.meta_type:
                continue
            ok = cond.verify_loop_guard(object, noacc=1)
            if ok:
                LOG('verify_back_guard', DEBUG, 'returning 1')
                return 1
            LOG('verify_back_guard', DEBUG, '%s no sig' % cond.getId())
        LOG('verify_back_guard', DEBUG, 'returning 0, no ok sig cond found')
        return 0

    # Verify a loop transition guard.
    # called by workflow guard
    security.declareProtected(View, 'verify_loop_guard')
    def verify_loop_guard(self, object, state_id, transition_id):
        LOG('verify_loop_guard', DEBUG, 'verifying state_id=%s transition_id=%s' % (state_id, transition_id))
        container = getattr(self, state_id).outgoing_conditions
        for cond in container.objectValues():
            if cond.get_transition_id() == transition_id:
                ok = cond.verify_loop_guard(object)
                LOG('verify_loop_guard', DEBUG, 'check cond = %s' % ok)
                return ok
        LOG('verify_loop_guard', DEBUG, 'guard not found state_id=%s transition_id=%s' % (state_id, transition_id))
        return 1

    security.declareProtected(View, 'get_variable')
    def get_variable(self, state_change, variable):
        LOG('get_variable', DEBUG, '%s -> %s' % (state_change, variable))
        if variable == 'schema_publication_lock':
            object = state_change.object
            # XXX locked by default
            # XXX find a way to explicitely set it at schema start
            return getattr(aq_base(object), variable, 1)
        if variable == 'PendingSignatures':
            object = state_change.object
            new_state = state_change.new_state
            if new_state is None: # initial state, default value
                old_state = state_change.old_state
                LOG('get_variable', DEBUG, 'pendingsignature new_state=None, ob=%s old_state=%s' % (object.getId(), old_state and old_state.getId() or 'None'))
                states = self.getSchemaStateIds()
                if not states:
                    return [] # no states at all !!!
                # the first state is the default state # XXX rather check in wf
                state_id = states[0]
            else:
                state_id = new_state.getId()
            container = getattr(self, state_id).outgoing_conditions
            pending = []
            for cond in container.objectValues():
                try:
                    issig = isinstance(cond, SignatureCondition)
                except TypeError:
                    # In python 2.1 isinstance() raises TypeError
                    # instead of returning 0 for ExtensionClasses.
                    issig = 0
                if not issig:
                    continue
                add = cond.get_pending_signatures(object)
                pending.extend(add)
            LOG('get_variable', DEBUG, 'pendingsignatures %s' % str(pending))
            return pending

    #
    # General workflow management
    #

    def generate_workflow(self, REQUEST=None):
        """Generates the core of the workflow."""
        self.delete_workflow()
        wf_id = self.getIdInTool()
        workflow_tool = getToolByName(self, 'portal_workflow')
        workflow_type_string = "dc_workflow (Web-configurable workflow)" # XXX may vary some day

        workflow_tool.manage_addWorkflow(workflow_type=workflow_type_string, id=wf_id)
        wf = getattr(workflow_tool, wf_id)
        variables = wf.variables
        scripts = wf.scripts


        variables.setStateVar('schema_state')

        variables.addVariable('schema_publication_lock')
        vdef = variables['schema_publication_lock']
        vdef.setProperties(description='',
                           for_catalog=1, for_status=0, update_always=1,
                           default_expr="python:scripts.get_variable(state_change, 'schema_publication_lock')")

        variables.addVariable('PendingSignatures')
        vdef = variables['PendingSignatures']
        vdef.setProperties(description='',
                           for_catalog=1, for_status=1, update_always=1,
                           default_expr="python:scripts.get_variable(state_change, 'PendingSignatures')")

        variables.addVariable('schema_name') # XXX used ?
        vdef = variables['schema_name']
        vdef.setProperties(description='',
                           for_catalog=1, for_status=0, update_always=0,
                           default_value=self.getId())

        variables.addVariable('schema_actor')
        vdef = variables['schema_actor']
        vdef.setProperties(description='',
                           for_catalog=0, for_status=1, update_always=1,
                           default_expr="user/getUserName")

        variables.addVariable('schema_time')
        vdef = variables['schema_time']
        vdef.setProperties(description='',
                           for_catalog=0, for_status=1, update_always=1,
                           default_expr="python:state_change.getDateTime().ISO()")

        variables.addVariable('schema_history')
        vdef = variables['schema_history']
        vdef.setProperties(description='',
                           for_catalog=0, for_status=0, update_always=0,
                           default_expr="state_change/getHistory")

        variables.addVariable('schema_action')
        vdef = variables['schema_action']
        vdef.setProperties(description='',
                           for_catalog=0, for_status=1, update_always=1,
                           default_expr="transition/getId | nothing")


        put_script(scripts, 'execute_transition')
        script = scripts['execute_transition']
        script.write("""\
##parameters=state_change
wf_id = state_change.workflow.id
schema = getattr(context.portal_workflow_schema, wf_id)
schema.execute_transition(state_change)
""")
        script._proxy_roles = ('Manager',)
        script._owner = None

        put_script(scripts, 'execute_loop_transition')
        script = scripts['execute_loop_transition']
        script.write("""\
##parameters=state_change
wf_id = state_change.workflow.id
schema = getattr(context.portal_workflow_schema, wf_id)
schema.execute_loop_transition(state_change)
""")
        script._proxy_roles = ('Manager',)
        script._owner = None

        put_script(scripts, 'verify_guard')
        script = scripts['verify_guard']
        script.write("""\
##parameters=object, workflow, state_id, transition_id
wf_id = workflow.id
schema = getattr(context.portal_workflow_schema, wf_id)
return schema.verify_guard(object, state_id, transition_id)
""")
        script._proxy_roles = ('Manager',)
        script._owner = None

        put_script(scripts, 'verify_back_guard')
        script = scripts['verify_back_guard']
        script.write("""\
##parameters=object, workflow, state_id, transition_id
wf_id = workflow.id
schema = getattr(context.portal_workflow_schema, wf_id)
return schema.verify_back_guard(object, state_id, transition_id)
""")
        script._proxy_roles = ('Manager',)
        script._owner = None

        put_script(scripts, 'verify_loop_guard')
        script = scripts['verify_loop_guard']
        script.write("""\
##parameters=object, workflow, state_id, transition_id
wf_id = workflow.id
schema = getattr(context.portal_workflow_schema, wf_id)
return schema.verify_loop_guard(object, state_id, transition_id)
""")
        script._proxy_roles = ('Manager',)
        script._owner = None

        put_script(scripts, 'get_variable')
        script = scripts['get_variable']
        script.write("""\
##parameters=state_change, variable
wf_id = state_change.workflow.id
schema = getattr(context.portal_workflow_schema, wf_id)
return schema.get_variable(state_change, variable)
""")
        script._proxy_roles = ('Manager',)
        script._owner = None

    def delete_workflow(self):
        """Deletes the workflow."""
        wf_id = self.getIdInTool()
        workflow_tool = getToolByName(self, 'portal_workflow')
        if wf_id in workflow_tool.objectIds():
            workflow_tool.manage_delObjects(ids=[wf_id])

    #def index_html(self,REQUEST):
    #    """ """
    #    return self.schema_view(self,REQUEST)



def addState(container, id, REQUEST=None, **kw):
    # container is a dispatcher when called from ZMI
    id = legal_id(id)
    ob = State(id, **kw)
    container._setObject(id, ob)
    ob = container._getOb(id)
    addEventContainer(ob, 'incoming_events')
    addEventContainer(ob, 'outgoing_events')
    addConditionContainer(ob, 'outgoing_conditions')
    addConditionContainer(ob, 'backward_conditions')
    ob.incoming_events.title = '_wfs_incoming_events_'
    ob.outgoing_events.title = '_wfs_outgoing_events_'
    ob.outgoing_conditions.title = '_wfs_conditions_'
    ob.backward_conditions.title = '_wfs_backward_conditions_'
    if REQUEST is not None:
        REQUEST.RESPONSE.redirect(container.absolute_url()+'/manage_main')
    else:
        return id

class State(PortalFolder, PortalContent, DefaultDublinCoreImpl):
    """
    Defines a Workflow Schema State.

    States are in lexical order.
    """
    meta_type = 'Workflow Schema State'
    portal_type = meta_type

    isPortalContent = 0

    security = ClassSecurityInfo()
    #security.declareObjectProtected(View)

    #security.declareObjectProtected(CMFCorePermissions.View)
    #security.declareObjectPublic()
    #security.setDefaultAccess("allow") # XXX why ?

    def __init__(self, id, title='', **kw):
        self.id = id
        self.title = title

    def manage_afterAdd(self, item, container):
        if aq_base(self) is not aq_base(container):
            self.add_state()
        State.inheritedAttribute('manage_afterAdd')(self, item, container)

    def manage_beforeDelete(self, item, container):
        State.inheritedAttribute('manage_beforeDelete')(self, item, container)
        if aq_base(self) is not aq_base(container):
            self.del_state()

    #def view(self,REQUEST):
    #    """ """
    #    return self.edit_state(self,REQUEST)
    #index_html=view

    #
    # ZMI
    #

    _properties = (
        {'id':'title', 'type':'string', 'mode':'w', 'label':'Title'},
        {'id':'back_transition_title', 'type':'string', 'mode':'w', 'label':'Returning Transition Title'},
        )
    back_transition_title = ''

    manage_options = PortalFolder.manage_options

    all_meta_types = (
        {'name': 'Workflow Schema Event Container',
         'action': 'manage_addEventContainerForm',
         'permission': AddWorkflowSchema,
         },
        {'name': 'Workflow Schema Condition Container',
         'action': 'manage_addConditionContainerForm',
         'permission': AddWorkflowSchema,
         },
        )

    manage_addEventContainerForm = DTMLFile('zmi/addEventContainerForm', globals())
    manage_addConditionContainerForm = DTMLFile('zmi/addConditionContainerForm', globals())

    security.declareProtected(AddWorkflowSchema, 'manage_addEventContainer')
    def manage_addEventContainer(self, id, REQUEST=None, **kw):
        """Add an Event Container from the ZMI."""
        if REQUEST is not None:
            kw.update(REQUEST.form)
        if kw.has_key('id'):
            del kw['id']
        addEventContainer(self, id, REQUEST=REQUEST, **kw)

    security.declareProtected(AddWorkflowSchema, 'manage_addConditionContainer')
    def manage_addConditionContainer(self, id, REQUEST=None, **kw):
        """Add a Condition Container from the ZMI."""
        if REQUEST is not None:
            kw.update(REQUEST.form)
        if kw.has_key('id'):
            del kw['id']
        addConditionContainer(self, id, REQUEST=REQUEST, **kw)

    #
    # Implementation
    #
    security.declareProtected(View, 'getState')
    def getState(self):
        return self

    # a prev transition goes from the previous state to us
    # a next transition goes from us to the next state
    def del_transitions(self, state_id, new_state_id, transition_id=None):
        """Delete transition from one state to another."""
        wf = self.getWorkflow()
        wf_transitions = wf.transitions
        state = wf.states.get(state_id)
        transitions = list(state.getTransitions())
        title = state.title
        if transition_id is None:
            transition_ids = list(transitions)
        else:
            transition_ids = [transition_id]
        for transition_id in transition_ids:
            if wf_transitions.get(transition_id).new_state_id == new_state_id:
                wf_transitions.deleteTransitions([transition_id])
                transitions.remove(transition_id)
                state.setProperties(title=title, transitions=transitions)

    def add_transition(self, state_id, new_state_id, transition_id,
                       label, url, trigger_type, props,
                       before_script, after_script):
        """Add transition from one state to another."""
        wf = self.getWorkflow()
        wf_transitions = wf.transitions
        state = wf.states.get(state_id)
        wf_transitions.addTransition(transition_id)
        transition = getattr(wf_transitions, transition_id)
        transition.setProperties(actbox_name=label,
                                 title='',
                                 actbox_url=url,
                                 actbox_category='schema',
                                 new_state_id=new_state_id,
                                 script_name=before_script,
                                 after_script_name=after_script,
                                 trigger_type=trigger_type,
                                 props=props,
                                 )
        # add transition to state
        title = state.title
        transitions = list(state.getTransitions())
        transitions.append(transition_id)
        state.setProperties(title=title, transitions=transitions)

    def change_transition(self, transition_id,
                          label=None):
        """Change properties of an existing transition."""
        wf = self.getWorkflow()
        wf_transitions = wf.transitions
        transition = getattr(wf_transitions, transition_id)
        if label is not None:
            transition.actbox_name = label

    def add_next_transitions(self, state_id, new_state_id):
        """Add next transitions to workflow."""
        transition_id = '%s_to_%s' % (state_id, new_state_id)
        label = 'Aller vers %s' % new_state_id
        url = '%(content_url)s/schema_do_action?action='+transition_id
        props = {'guard_expr': 'python:scripts.verify_guard(here, workflow, "%s", "%s")' % (state_id, transition_id)}
        trigger_type = 0 # automatic
        before_script = 'execute_transition'
        after_script = ''
        self.add_transition(state_id, new_state_id, transition_id,
                            label, url, trigger_type, props,
                            before_script, after_script)

    def add_prev_transitions(self, state_id, new_state_id):
        """Add prev transitions to workflow."""
        transition_id = '%s_to_%s' % (state_id, new_state_id)
        label = (self.getWorkflowSchema()[new_state_id].back_transition_title
                 or 'Back to %s' % new_state_id)
        url = '%(content_url)s/schema_do_action?action='+transition_id
        props = {'guard_expr': 'python:scripts.verify_back_guard(here, workflow, "%s", "%s")' % (state_id, transition_id)}
        before_script = 'execute_transition'
        after_script = ''
        trigger_type = 1 # manual
        self.add_transition(state_id, new_state_id, transition_id,
                            label, url, trigger_type, props,
                            before_script, after_script)

    def add_state(self):
        """
        Called when the state is added or inserted to the schema.

        When inserting, we must regenerate some transitions to the
        workflow.
        """
        wf_states = self.getWorkflow().states
        state_id = self.getId()
        s_state_ids = self.getSchemaStateIds() # ordered
        wf_state_ids = self.getWorkflowStateIds() # ordered
        pos = s_state_ids.index(state_id)

        # find previous
        prev_id = None
        r = range(0, pos)
        r.reverse()
        for i in r:
            id = s_state_ids[i]
            if id in wf_state_ids:
                prev_id = id
                break
        # find next
        next_id = None
        r = range(pos, len(s_state_ids))
        for i in r:
            id = s_state_ids[i]
            if id in wf_state_ids:
                next_id = id
                break

        # do insertion

        if prev_id is not None and next_id is not None:
            self.del_transitions(prev_id, next_id)
            self.del_transitions(next_id, prev_id)

        wf_states.addState(state_id)

        if prev_id is not None:
            self.add_next_transitions(prev_id, state_id)
            self.add_prev_transitions(state_id, prev_id)
        if next_id is not None:
            self.add_next_transitions(state_id, next_id)
            self.add_prev_transitions(next_id, state_id)

        if prev_id is None:
            wf_states.setInitialState(state_id)

    def del_state(self):
        """
        Called when a state is deleted.
        """
        wf_states = self.getWorkflow().states
        state_id = self.getId()
        s_state_ids = self.getSchemaStateIds() # ordered
        wf_state_ids = self.getWorkflowStateIds() # ordered list
        pos = s_state_ids.index(state_id)

        # find previous
        prev_id = None
        r = range(0, pos)
        r.reverse()
        for i in r:
            id = s_state_ids[i]
            if id in wf_state_ids:
                prev_id = id
                break
        # find next
        next_id = None
        r = range(pos+1, len(s_state_ids))
        for i in r:
            id = s_state_ids[i]
            if id in wf_state_ids:
                next_id = id
                break

        # do removal

        if prev_id is not None:
            self.del_transitions(prev_id, state_id)
            self.del_transitions(state_id, prev_id)
        if next_id is not None:
            self.del_transitions(next_id, state_id)
            self.del_transitions(state_id, next_id)

        # XXX may raise KeyError
        wf_states.deleteStates([state_id])

        if prev_id is not None and next_id is not None:
            self.add_next_transitions(prev_id, next_id)
            self.add_prev_transitions(next_id, prev_id)

        if prev_id is None and next_id is not None:
            wf_states.setInitialState(next_id)

    def set_permissions(self, ob):
        """
        Set the permissions or local roles on the object.
        """
        container = self.outgoing_conditions
        for cond in container.objectValues():
            cond.set_permissions(ob)



#
# Events
#
def addEventContainer(container, id, REQUEST=None, **kw):
    # container is a dispatcher when called from ZMI
    ob = EventContainer(id, **kw)
    container._setObject(id, ob)
    if REQUEST is not None:
        REQUEST.RESPONSE.redirect(container.absolute_url()+'/manage_main')

class EventContainer(PortalFolder, PortalContent, DefaultDublinCoreImpl):
    """
    Contains Events.
    """
    meta_type = 'Workflow Schema Event Container'
    portal_type = meta_type

    security = ClassSecurityInfo()

    _properties = (
        {'id':'title', 'type':'string', 'mode':'w', 'label':'Title'},
        {'id':'description', 'type':'text', 'mode':'w', 'label':'Description'},
        )

    def __init__(self, id, title='', description='', **kw):
        self.id = id
        self.title = title
        self.description = description

    #
    # ZMI
    #
    all_meta_types = (
        {'name': 'Workflow Schema Mail Event',
         'action': 'manage_addMailEventForm',
         'permission': AddWorkflowSchema,
         },
        {'name': 'Workflow Schema Publication Switch Event',
         'action': 'manage_addPublicationSwitchEventForm',
         'permission': AddWorkflowSchema,
         },
        )

    manage_addMailEventForm = DTMLFile('zmi/addMailEventForm', globals())
    manage_addPublicationSwitchEventForm = DTMLFile('zmi/addPublicationSwitchEventForm', globals())

    security.declareProtected(AddWorkflowSchema, 'manage_addMailEvent')
    def manage_addMailEvent(self, REQUEST=None, **kw):
        """Add a Mail Event from the ZMI."""
        if REQUEST is not None:
            kw.update(REQUEST.form)
        id = 'event_' + str(random.randrange(1, 10000000))
        addMailEvent(self, id, REQUEST=REQUEST, **kw)
        return id

    security.declareProtected(AddWorkflowSchema, 'manage_addPublicationSwitchEvent')
    def manage_addPublicationSwitchEvent(self, REQUEST=None, **kw):
        """Add a PublicationSwitch Event from the ZMI."""
        if REQUEST is not None:
            kw.update(REQUEST.form)
        id = 'event_' + str(random.randrange(1, 10000000))
        addPublicationSwitchEvent(self, id, REQUEST=REQUEST, **kw)
        return id

def addMailEvent(container, id, REQUEST=None, **kw):
    # container is a dispatcher when called from ZMI
    ob = MailEvent(id)
    container._setObject(id, ob)
    ob = container._getOb(id)
    ob.edit_properties(**kw)
    if REQUEST is not None:
        REQUEST.RESPONSE.redirect(container.absolute_url()+'/manage_main')

def addPublicationSwitchEvent(container, id, REQUEST=None, **kw):
    # container is a dispatcher when called from ZMI
    ob = PublicationSwitchEvent(id)
    container._setObject(id, ob)
    ob = container._getOb(id)
    ob.edit_properties(**kw)
    if REQUEST is not None:
        REQUEST.RESPONSE.redirect(container.absolute_url()+'/manage_main')

class BasicEvent(PortalContent, DefaultDublinCoreImpl):
    """
    Specific scenarios (mailing, permission changing, etc) are to be derived
    from this class. Each scenario can be instantiated and parametered.
    """
    meta_type = None

    isPortalContent = 0

    _properties = (
        {'id':'title', 'type':'string', 'mode':'w', 'label':'Title'},
        {'id':'description', 'type':'text', 'mode':'w', 'label':'Description'},
        )

    security = ClassSecurityInfo()

    manage_options = (
        PropertyManager.manage_options +   # Properties
        PortalContent.manage_options[:1] + # skip Edit
        PortalContent.manage_options[3:]
        )

    def __init__(self, id, **kw):
        self.id = id
        self.description = kw.get('description') or ''

    def manage_afterAdd(self, item, container):
        if aq_base(self) is not aq_base(container):
            pass
        BasicEvent.inheritedAttribute('manage_afterAdd')(self, item, container)

    def manage_beforeDelete(self, item, container):
        BasicEvent.inheritedAttribute('manage_beforeDelete')(self, item, container)
        if aq_base(self) is not aq_base(container):
            pass

    #
    # Used by PropertyManager
    #
    security.declarePublic('propertyLabel')
    def propertyLabel(self, id):
        """Return a label for the given property id."""
        for p in self._properties:
            if p['id'] == id:
                return p.get('label', id)
        return id

    #
    # Accessors
    #
    security.declareProtected(AddWorkflowSchema, 'edit_properties')
    def edit_properties(self, REQUEST=None, **kw):
        if REQUEST is not None:
            kw.update(REQUEST.form)
        self.manage_changeProperties(**kw)
        if REQUEST is not None and kw.get('came_from', 0):
            REQUEST.RESPONSE.redirect(kw['came_from'])

    #
    # CMF
    #
    security.declareProtected(View, 'edit_form')
    def edit_form(self):
        """Return the edit form."""
        method = _getViewFor(self, view='edit')
        if getattr(aq_base(method), 'isDocTemp', 0):
            return method(self, self.REQUEST)
        else:
            return method()


class MailEvent(BasicEvent):
    """Mail Event Class."""
    meta_type = 'Workflow Schema Mail Event'
    portal_type = meta_type

    security = ClassSecurityInfo()
    security.declareObjectProtected(View)

    _properties = BasicEvent._properties + (
        {'id':'mailinglist', 'type':'string', 'mode':'w', 'label':'Mailing list'},
        {'id':'from_address', 'type':'string', 'mode':'w', 'label':'From address'},
        {'id':'subject', 'type':'string', 'mode':'w', 'label':'Subject'},
        {'id':'body', 'type':'text', 'mode':'w', 'label':'Message'},
        )

    def __init__(self, id):
        BasicEvent.__init__(self, id)
        self.mailinglist = 'mailing'
        self.from_address = ''
        self.subject = 'No subject'
        self.body = 'No body'

    def cooked_from_address(self,object):
        """Get from address.

        Evaluates DTML, if empty then get current user's.
        """
        address = HTML(self.from_address)(object).strip()
        if not address:
            me = self.REQUEST['AUTHENTICATED_USER'].getUserName()
            entry = self.portal_metadirectories.members.getEntry(me)
            if not entry:
                return 'nobody@example.com'
            address = entry.get('email')
            if not address:
                return 'nobody@example.com'
        return address

    def cooked_body(self,object):
        """ """
        return HTML(self.body)(object)

    def cooked_subject(self,object):
        """ """
        return HTML(self.subject)(object)

    def execute_event(self, object):
        #LOG('execute_event', DEBUG, '=> %s (%s)' % (state_change.new_state, state_change.transition.getId()))
        dtool = self.portal_metadirectories
        name = self.mailinglist
        mailing = None
        if name == '_mailing_author':
            mailing = name
        else:
            for context in (object, self):
                ob = getattr(context, name, None)
                if getattr(ob, 'meta_type', None) != 'Mailing List':
                    continue
                mailing = ob
                break
        if mailing is None:
            LOG('WorkflowSchema', PROBLEM, "Cannot find mailing list %s for Mailing Event %s in Schema %s" % (name, self.title_or_id(), self.getWorkflowSchema().title_or_id()))
            return
        LOG('WorkflowSchema', DEBUG, "Sending to mailing list %s for Mailing Event %s in Schema %s" % (name, self.title_or_id(), self.getWorkflowSchema().title_or_id()))

        from_address = self.cooked_from_address(object)
        subject = self.cooked_subject(object)
        body = self.cooked_body(object)

        if mailing == '_mailing_author':
            creator = object.Creator()
            if not creator or creator == 'No owner':
                return
            entry = dtool.members.getEntry(creator)
            if not entry:
                return
            mto = entry.get('email')
            if not mto:
                return
            content = mail_send_template % (from_address,
                                            mto,
                                            subject,
                                            body)
            self.MailHost.send(content,
                               mto=mto,
                               mfrom=from_address,
                               subject=subject,
                               encode='8bit')
        else:
            mailing.send_email(from_address=from_address,
                               subject=subject,
                               body=body,
                               context=object)

class PublicationSwitchEvent(BasicEvent):
    """
    PublicationSwitch Event Class.
    Enables or disables publication.
    """
    meta_type = 'Workflow Schema Publication Switch Event'
    portal_type = meta_type

    security = ClassSecurityInfo()
    security.declareObjectProtected(View)

    _properties = BasicEvent._properties + (
        {'id':'allow', 'type':'boolean', 'mode':'w', 'label':'Allow publication'},
        {'id':'leave_schema', 'type':'boolean', 'mode':'w', 'label':'Leave the schema'},
        )
    allow = 1
    leave_schema = 0

    def __init__(self, id):
        BasicEvent.__init__(self, id)

    def execute_event(self, object):
##         if not hasattr(object, 'setWorkflowAddChain'):
##             return
##         workflow_id = 'publication_workflow'
##         chain = list(object.getWorkflowAddChain())
##         if self.allow:
##             if workflow_id not in chain:
##                 chain.append(workflow_id)
##                 object.setWorkflowAddChain(chain)
##         else:
##             if workflow_id in chain:
##                 chain.remove(workflow_id)
##                 object.setWorkflowAddChain(chain)
        # The publication workflow checks this lock before
        # allowing a publication guard.
        object.schema_publication_lock = not self.allow
        if hasattr(object, 'setWorkflowIgnoreChain'):
            if self.leave_schema:
                wf_id = self.getIdInTool()
                ichain = object.getWorkflowIgnoreChain()+(wf_id,)
                object.setWorkflowIgnoreChain(ichain)
                LOG('WorkflowSchema', DEBUG, 'setting wf_ignore=%s for ob=%s' % (str(ichain), object.getId()))


#
# Conditions
#
def addConditionContainer(container, id, REQUEST=None, **kw):
    # container is a dispatcher when called from ZMI
    ob = ConditionContainer(id, **kw)
    container._setObject(id, ob)
    if REQUEST is not None:
        REQUEST.RESPONSE.redirect(container.absolute_url()+'/manage_main')

class ConditionContainer(PortalFolder, PortalContent, DefaultDublinCoreImpl):
    """
    Contains Conditions.
    """
    meta_type = 'Workflow Schema Condition Container'
    portal_type = meta_type

    security = ClassSecurityInfo()

    _properties = (
        {'id':'title', 'type':'string', 'mode':'w', 'label':'Title'},
        {'id':'description', 'type':'text', 'mode':'w', 'label':'Description'},
        )

    def __init__(self, id, title='', description='', **kw):
        self.id = id
        self.title = title
        self.description = description
        self.acquired_required_signatures=[] # XXX needed here ?

    #
    # ZMI
    #
    all_meta_types = (
        {'name': 'Workflow Schema Signature Condition',
         'action': 'manage_addSignatureConditionForm',
         'permission': AddWorkflowSchema,
         },
        {'name': 'Workflow Schema Substate Condition',
         'action': 'manage_addSubstateConditionForm',
         'permission': AddWorkflowSchema,
         },
        )

    manage_addSignatureConditionForm = DTMLFile('zmi/addSignatureConditionForm', globals())
    manage_addSubstateConditionForm = DTMLFile('zmi/addSubstateConditionForm', globals())

    security.declareProtected(AddWorkflowSchema, 'manage_addSignatureCondition')
    def manage_addSignatureCondition(self, REQUEST=None, **kw):
        """Add a Signature Condition from the ZMI."""
        if REQUEST is not None:
            kw.update(REQUEST.form)
        id = 'cond_' + str(random.randrange(1, 10000000))
        addSignatureCondition(self, id, REQUEST=REQUEST, **kw)
        return id

    security.declareProtected(AddWorkflowSchema, 'manage_addSubstateCondition')
    def manage_addSubstateCondition(self, REQUEST=None, **kw):
        """Add a Substate Condition from the ZMI."""
        if REQUEST is not None:
            kw.update(REQUEST.form)
        id = 'cond_' + str(random.randrange(1, 10000000))
        addSubstateCondition(self, id, REQUEST=REQUEST, **kw)
        return id

def addSignatureCondition(container, id, REQUEST=None, **kw):
    # container is a dispatcher when called from ZMI
    ob = SignatureCondition(id)
    # This must be done before setting because manage_afterAdd uses the title
    #ob.edit_properties(**kw)
    # XXX But edit_properties->manage_afterPropertyChange -> needs wrapper
    # XXX so hack the title in...
    ob.title = kw.get('title', '') # XXX ugh
    container._setObject(id, ob)
    ob = container._getOb(id)
    ob.edit_properties(**kw)
    if REQUEST is not None:
        REQUEST.RESPONSE.redirect(container.absolute_url()+'/manage_main')

def addSubstateCondition(container, id, REQUEST=None, **kw):
    # container is a dispatcher when called from ZMI
    ob = SubstateCondition(id)
    ob.title = kw.get('title', '') # XXX ugh
    container._setObject(id, ob)
    ob = container._getOb(id)
    ob.edit_properties(**kw)
    if REQUEST is not None:
        REQUEST.RESPONSE.redirect(container.absolute_url()+'/manage_main')

class BasicCondition(PortalContent, DefaultDublinCoreImpl):
    """
    Base class for a condition.

    A condition adds:
     - a transition to the next state, with a guard
     - optionnaly, a loop transition to change state
    """
    meta_type = None

    isPortalContent = 0

    _properties = (
        {'id':'title', 'type':'string', 'mode':'w', 'label':'Title'},
        {'id':'description', 'type':'text', 'mode':'w', 'label':'Description'},
        )

    security = ClassSecurityInfo()

    manage_options = (
        PropertyManager.manage_options +   # Properties
        PortalContent.manage_options[:1] + # skip Edit
        PortalContent.manage_options[3:]
        )

    def __init__(self, id, **kw):
        self.id = id
        self.description = ""
        self.exit_states = []
        self.enter_states = []
        self.conditioned_states = []

    def manage_afterAdd(self, item, container):
        if aq_base(self) is not aq_base(container):
            self.add_loop_transition()
        BasicCondition.inheritedAttribute('manage_afterAdd')(self, item, container)

    def manage_beforeDelete(self, item, container):
        BasicCondition.inheritedAttribute('manage_beforeDelete')(self, item, container)
        if aq_base(self) is not aq_base(container):
            self.del_loop_transition()

    def manage_afterPropertyChange(self, id, old_value):
        if id in ['title']:
            self.update_loop_transition()

    #
    # Used by PropertyManager
    #
    security.declarePublic('propertyLabel')
    def propertyLabel(self, id):
        """Return a label for the given property id."""
        for p in self._properties:
            if p['id'] == id:
                return p.get('label', id)
        return id

    #
    # Accessors
    #
    security.declareProtected(AddWorkflowSchema, 'edit_properties')
    def edit_properties(self, REQUEST=None, **kw):
        if REQUEST is not None:
            kw.update(REQUEST.form)
        self.manage_changeProperties(**kw)
        if REQUEST is not None and kw.get('came_from', 0):
            REQUEST.RESPONSE.redirect(kw.get('came_from'))

    #
    # CMF
    #
    security.declareProtected(View, 'edit_form')
    def edit_form(self):
        """Return the edit form."""
        method = _getViewFor(self, view='edit')
        if getattr(aq_base(method), 'isDocTemp', 0):
            return method(self, self.REQUEST)
        else:
            return method()

    #
    # Support methods
    #
    def add_loop_transition(self):
        """Add loop transition to workflow."""
        state = self.getState()
        state_id = state.getId()
        new_state_id = '' # remain in state
        wf = self.getWorkflow()
        transition_id = self.get_transition_id()
        label = self.get_transition_title()
        url = '%(content_url)s/schema_do_action?action='+transition_id
        props = {
            'guard_expr': 'python:scripts.verify_loop_guard(here, workflow, "%s", "%s")' % (state_id, transition_id),
            'category': 'schema',
            }
        # before_script is used so that the transition can update variables,
        # which will be taken into account.
        before_script = 'execute_loop_transition'
        after_script = ''
        trigger_type = 1 # manual
        state.add_transition(state_id, new_state_id, transition_id,
                             label, url, trigger_type, props,
                             before_script, after_script)

    def del_loop_transition(self):
        """Delete loop transition from workflow."""
        state = self.getState()
        state_id = state.getId()
        new_state_id = ''
        transition_id = self.get_transition_id()
        state.del_transitions(state_id, new_state_id, transition_id=transition_id)

    def update_loop_transition(self):
        """Update a loop transition when properties have changed."""
        state = self.getState()
        transition_id = self.get_transition_id()
        label = self.get_transition_title()
        state.change_transition(transition_id, label=label)

    def get_transition_id(self):
        state_id = self.getState().getId()
        cond_id = self.getId()
        transition_id = '%s_loop_%s' % (state_id, cond_id)
        return transition_id

    def get_transition_title(self):
        return self.title

    def set_permissions(self, ob):
        """
        Set the permissions or local roles on the object.
        """
        pass # FIXME


class SubstateCondition(BasicCondition):
    """
    A Substate Condition is passed if some subobjects are in a certain state.
    """
    meta_type = 'Workflow Schema Substate Condition'
    portal_type = meta_type

    _properties = BasicCondition._properties + (
        {'id':'workflow_variable', 'type':'string', 'mode':'w', 'label':'Workflow variable'},
        {'id':'required_state', 'type':'string', 'mode':'w', 'label':'Required state'},
        {'id':'filtered_portal_types', 'type':'lines', 'mode':'w', 'label':'Objects Portal Types'},
        )

    def __init__(self, id):
        BasicCondition.__init__(self, id)
        self.supporting_transitions = ()
        self.workflow_variable = 'review_state'
        self.required_state = 'published'
        self.filtered_portal_types = []

    def cleanup(self, object):
        object._manager_explicitly_bypassed = 0

    def verify_guard(self, object):
        LOG('verify_guard', DEBUG, "dans verify_guard")
        if getattr(object, '_manager_explicitly_bypassed', 0):
            LOG('verify_guard', DEBUG, "getattr(object, '_manager_explicitly_bypassed', 0) -> 1")
            return 1
        workflow_tool = getToolByName(self, 'portal_workflow')
        for child in object.objectValues():
            LOG('verify_guard', DEBUG, 'child=%s' % child.getId())
            pt = getattr(aq_base(child), 'portal_type', None)
            LOG('verify_guard', DEBUG, 'pt=%s' % pt)
            LOG('verify_guard', DEBUG, "self.filtered_portal_types is %s" % self.filtered_portal_types)
            if pt in self.filtered_portal_types:
                LOG('verify_guard', DEBUG, "pt in self.filtered_portal_types")
                child_review_state = workflow_tool.getInfoFor(child, self.workflow_variable, None)
                LOG('verify_guard', DEBUG, "child_review_state=%s" % child_review_state)
                if child_review_state != self.required_state:
                    LOG('verify_guard', DEBUG, "%s==?%s" % (child_review_state, self.required_state))
                    return 0
        return 1

    def get_transition_title(self): # XXX ???
        title = self.title or 'Bypass sub-element restrictions'
        return title

    def verify_loop_guard(self, object):
        """Decide who can bypass sub-elements."""
        is_allowed= self.REQUEST['AUTHENTICATED_USER'].has_role('Manager')
        not_bypassed=not getattr(aq_base(self),'_manager_explicitly_bypassed',None)
        condition_relevent=not self.verify_guard(object)
        return is_allowed and not_bypassed and condition_relevent

    # not used FIXME
    def get_status_text(self,object):
        """ """
        if self.verify_guard(object):
            return "%s (%s) condition is blocking publication"%(self.getId(),self.title)
        else:
            return "%s (%s) condition met"%(self.getId(),self.title)

    def execute_event(self,object):
        """ """
        object._manager_explicitly_bypassed=1


class SignatureCondition(BasicCondition):
    """
    A Signature Condition is passed if any/all parties have signed.
    """
    # XXX deal with groups !

    meta_type = 'Workflow Schema Signature Condition'
    portal_type = meta_type

    accumulated_signatures = []
    required_signatures = []
    acquired_required_signatures = []
    one_enough = 1 # 0 = AND, 1 = OR

    _properties = BasicCondition._properties + (
        {'id':'required_signatures', 'type':'lines', 'mode':'w', 'label':'Required signatures'},
        {'id':'acquired_required_signatures', 'type':'lines', 'mode':'w', 'label':'Ids of signature lists to acquire'},
        {'id':'one_enough', 'type':'boolean', 'mode':'w', 'label':'One signature is enough'},
        )

    security = ClassSecurityInfo()

    def __init__(self, id):
        BasicCondition.__init__(self, id)

    def cleanup(self, object):
        object.accumulated_signatures = []


    # move this elsewhere XXX
    def get_group_users(self, gid):
        aclu = self.acl_users
        if not hasattr(aclu, 'getGroupById'):
            return []
        group = aclu.getGroupById(gid, None)
        if group is None:
            return []
        return group.getUsers()

    def get_role_users(self, role, object):
        ids = []
        merged = mergedLocalRoles(object, withgroups=1)
        for id, roles in merged.items():
            if role in roles:
                ids.append(id)
        LOG('SignatureCondition', DEBUG, 'get_role_users role=%s object=%s'
            ' -> %s' % (role, '/'.join(object.getPhysicalPath()), merged))
        return self.expand_groups(ids, object)

    def expand_groups(self, ids, object):
        """Expand special ids for signatures into user ids.

        Recognises 'group:foo' and 'role:foo'. The latter only works
        for local roles, not global roles.

        Also recognises 'user:foo'.
        """
        okids = []
        for id in ids:
            id = id.strip()
            if id.startswith('group:'):
                gid = id[len('group:'):]
                okids.extend(self.get_group_users(gid))
            elif id.startswith('role:'):
                role = id[len('role:'):]
                okids.extend(self.get_role_users(role, object))
            elif id.startswith('user:'):
                uid = id[len('user:'):]
                okids.append(uid)
            elif id:
                okids.append(id)
        LOG('SignatureCondition', DEBUG, 'expand_groups ids=%s -> %s'
            % (ids, okids))
        return okids

    def get_required_signatures(self, object):
        sigs = self.required_signatures
        for list_id in self.acquired_required_signatures:
            if not list_id.startswith('_'):
                sigs = sigs + list(getattr(self, list_id, []))
        res = self.expand_groups(sigs, object)
        return res

    def get_pending_signatures(self, object):
        """Called by get_variable for PendingSignatures."""
        req_sigs = self.get_required_signatures(object)
        sigs = getattr(object, 'accumulated_signatures', [])
        pending = []
        for name in req_sigs:
            if name != '' and name not in sigs:
                pending.append(name)
        return pending

    # This checks if a loop in the same state is allowed
    # meaning, we can sign the document
    def verify_loop_guard(self, object, noacc=0):
        req_sigs = self.get_required_signatures(object)
        if noacc:
            # Permit testing without regard to accumulated sigs.
            sigs = []
        else:
            sigs = getattr(object, 'accumulated_signatures', [])
        # XXX use getSecurity.getUser ?
        name = self.REQUEST['AUTHENTICATED_USER'].getUserName()
        if name in req_sigs and name not in sigs:
            return 1
        return 0

    # This checks if we can now go to the next state
    def verify_guard(self, object):
        req_sigs = self.get_required_signatures(object)
        sigs = getattr(object, 'accumulated_signatures', [])
        if not self.one_enough: # AND
            ok = 1
            for name in req_sigs:
                if name != '' and name not in sigs:
                    ok = 0
                    break
        else: # OR
            ok = 0
            for name in req_sigs:
                if name != '' and name in sigs:
                    ok = 1
                    break
        LOG('sig verify_guard', DEBUG, 'returning ok=%s' % ok)
        return ok


    def execute_event(self, object):
        name = self.REQUEST['AUTHENTICATED_USER'].getUserName()
        sigs = getattr(object, 'accumulated_signatures', [])
        if name not in sigs:
            sigs.append(name)
            object.accumulated_signatures = sigs
            #object.reindexObject() # to update PendingSignatures

    def set_permissions(self, ob):
        """
        Set the permissions or local roles on the object.
        """
        # XXX must give Reader to:
        # - Manager
        # - users or groups in allowed_signatures
        pass


InitializeClass(WorkflowSchema)
InitializeClass(State)
InitializeClass(EventContainer)
InitializeClass(ConditionContainer)
InitializeClass(BasicEvent)
InitializeClass(MailEvent)
InitializeClass(PublicationSwitchEvent)
InitializeClass(BasicCondition)
InitializeClass(SubstateCondition)
InitializeClass(SignatureCondition)

