# (c) 2002 Prfecture du Bas-Rhin, France
# (c) 2002 Adelux <http://adequat.c2a.fr>
# (c) 2002 Nuxeo SARL <http://nuxeo.com>
# (c) 2002 Florent Guillaume <mailto:fg@nuxeo.com>
# See license info at the end of this file.
# $Id: WorkflowSchemaTool.py,v 1.5 2002/12/04 18:30:23 jalon Exp $

"""
  Workflow Schema Tool
"""

__version__='$Revision: 1.5 $'[11:-2]

import string
from zLOG import LOG, DEBUG
from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo

from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFCore.CMFCorePermissions import View
from Products.CMFCore.utils import UniqueObject, getToolByName

from WorkflowSchemaPermissions import AddWorkflowSchema
from WorkflowSchema import Schema_Type, addWorkflowSchema

## def addWorkflowSchemaTool(container, id, REQUEST=None):
##     """Add a Workflow Schema Tool."""
##     obj = WorkflowSchemaTool(id)
##     container._setObject(id, obj)
##     if REQUEST is not None:
##         REQUEST.RESPONSE.redirect(container.absolute_url()+'/manage_main')


class WorkflowSchemaTool(UniqueObject, PortalFolder):
    """
    Workflow Schema Tool.
    """
    id = 'portal_workflow_schema'
    meta_type = 'Workflow Schema Tool'
    portal_type = 'Workflow Schema Tool'

    security = ClassSecurityInfo()

    def __init__(self):
        PortalFolder.__init__(self, self.id)
        self.setTitle('_wfs_Workflow_Schemas_')

    #
    # ZMI methods
    #
    all_meta_types = (
        {'name': Schema_Type,
         'action': 'manage_addWorkflowSchemaForm',
         'permission': AddWorkflowSchema,
         },
        )

    manage_addWorkflowSchemaForm = DTMLFile('zmi/addWorkflowSchemaForm', globals())

    security.declareProtected(AddWorkflowSchema, 'manage_addWorkflowSchema')
    def manage_addWorkflowSchema(self, id,
                                 title='',
                                 description='',
                                 REQUEST=None, **kw):
        """Add a Workflow Schema from the ZMI."""
        container = self
        id = addWorkflowSchema(container, id,
                               title=title,
                               description=description,
                               REQUEST=REQUEST)
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(container.absolute_url() 
                + '/%s/manage_workspace' % id)

    #
    # Object association
    #
    security.declarePublic('associateObject')
    def associateObject(self, object, container=None, wfs_id=None):
        """
        Associate the object with a given workflow schema.

        Return true if the association was made.
        """
        wfs_ids = self.getSchemasFor(object=object, container=container)
        LOG('associateObject', DEBUG, 'ob = %s, got wfs_ids %s' % (object.getId(), str(wfs_ids)))
        if wfs_ids:
            if wfs_id is None:
                wfs_id = wfs_ids[0] # use first one
        if wfs_id not in wfs_ids:
            return 0
        wf_id = getattr(self, wfs_id).getIdInTool()
        if hasattr(object, 'setWorkflowAddChain'):
            chain = list(object.getWorkflowAddChain())
            if wf_id not in chain:
                for id in list(chain):
                    if id.startswith('schema_'): # XXX hardcoded prefix
                        chain.remove(id)
                chain.append(wf_id)
                object.setWorkflowAddChain(chain)
                # ensure wf variables (PendingSignatures) are updated
                object.reindexObject()
                return 1
        return 0

    #
    # Public methods
    #
    security.declarePublic('getSchemasFor')
    def getSchemasFor(self, portal_type=None, object=None, container=None):
        """
        Get the schema ids with which a portal_type can be created
        or which an object can follow.

        Two things are consulted: an acquired property named
        workflow_schema_associations, containing a list of lines
        of the form "My Portal Type:wfs_id",
        and the global association list of the workflow schemas.
        """
        if object is not None:
            portal_type = object.portal_type
        ok = []
        wfs_items = self.objectItems(Schema_Type)
        wfs_ids = self.objectIds(Schema_Type)
        # first try contextual types
        if container is not None:
            # acquire
            assoc = getattr(container, 'workflow_schema_associations', None)
            # is a list of lines, like "My Portal Type:wfs_id"
            if assoc is not None:
                for line in assoc:
                    lines = line.split(':')
                    if len(lines) == 2:
                        pt, wfs_id = lines
                        if pt == portal_type and wfs_id in wfs_ids:
                            if wfs_id not in ok:
                                ok.append(wfs_id)
        # try types from workflow schemas
        for (wfs_id, wfs) in wfs_items:
            types = wfs.getProperty('associated_portal_types', [])
            for pt in types:
                pt = pt.strip() # this comes from a "lines" property
                if portal_type == pt:
                    if wfs_id not in ok:
                        ok.append(wfs_id)
        return ok

    #
    # CMF views
    #
    def __call__(self, REQUEST=None, **kw):
        """Default view."""
        return self.view(REQUEST=REQUEST, **kw)

    index_html = None  # This special value informs ZPublisher to use __call__

    security.declareProtected(View, 'view')
    def view(self, REQUEST=None, **kw):
        """Default view."""
        return self.wfschema_tool_view(REQUEST=REQUEST, **kw)


InitializeClass(WorkflowSchemaTool)


# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
